/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.storage.impl;

import java.util.*;

import anthill.Resource;
import anthill.factory.Factory;
import anthill.storage.*;

/**
 *  Implementation of an LRU based query storage.  This is a wrapper class
 *  for storing resources based on using a query to access the resources.
 *  It inherits all methods from the <CODE>LRUKeyStorage</CODE>,
 *  but adds a method to retrieve resources based on a query.
 *
 *  @author Hein Meling
 *  @version $Revision: 1.6 $
 */
public class LRUQueryStorage
extends LRUKeyStorage
implements QueryStorage
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Constant null array to be returned for the case of no resources
   *  (object reuse).  See Item 27 in Effective Java.
   */
  private final static Resource[] NULL_RESOURCE_ARRAY = new Resource[0];


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   *  The identifier of the configuration parameter 
   */
  public static final String PAR_CONTROLLERNAME = "ControllerName";


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** The query controller provided by the implementing class. */
  private QueryController queryCtrl;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Contructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * 
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public LRUQueryStorage(Factory factory, String name)
  {
    this.factory = factory;
    this.name = name;
    clear();
  }

  /**
   *  Restore the object to its initial state
   */
  public void clear()
  {
    super.clear();
    String controllerName = factory.stringValue(name + "." + PAR_CONTROLLERNAME);
    queryCtrl = (QueryController) factory.get(controllerName);
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Public methods for LRUQueryStorage
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Return the resource associated with the specified query.
   *
   *  @param query  whose associated resource to retreive.
   *  @return resources found in the storage with which the
   *    specified query matched.
   */
  public Resource[] getResources(Object query)
  {
    List resources = new ArrayList();
    for (Iterator iter = storage.iterator(); iter.hasNext(); ) {
      Resource resource = (Resource) iter.next();
      if (queryCtrl.satisfies(resource, query))
        resources.add(resource);
    }
    return (Resource[]) resources.toArray(NULL_RESOURCE_ARRAY);
  }

} // END LRUQueryStorage
