/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.types;

import anthill.*;
import anthill.factory.Factory;

/**
 * 
 *  @author Alberto Montresor
 *  @version $Revision: 1.3 $
 */
public class ArrayReturnPath
implements ReturnPath
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** The array containing the path of visited nests */
  private NestId[] nests;
  
  /** The current length of the path */
  private int size;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Constructs a new empty return path. Actual initialization will be 
   *  postponed to method init, when a view is available. This constructor
   *  is used by the factory mechanism.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public ArrayReturnPath(Factory factory, String name)
  {
  }
  
  /**
   *  Constructs a new empty return path. Actual initialization will be 
   *  postponed to method init, when a view is available.
   */
  public ArrayReturnPath()
  {
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Initializes this data structure. This method must be invoked
   *  at most once, for example in the <CODE>init</CODE> method of
   *  ants.
   *  <BR>
   *  The initialization is performed by creating an array capable
   *  to contain TTL/2 elements.
   *  
   *  
   *  @param view the current view associated to the ant using this
   *   data structure
   */
  public void init(AntView view)
  {
    int maxsize = view.getTTL();
    nests = new NestId[maxsize / 2];
    size = 0;
  }
  
  /**
   *  Adds a new element to the path. Returns true if this element
   *  already belongs to the path, false otherwise. Depending on
   *  the implementation, if the element already belongs to the
   *  path, the element may be added again or not.
   *  <BR>
   *  The element is searched by performing a linear search in the
   *  array. If found, true is returned. In any case, the element
   *  is added to the path.
   *  
   *  @param view the current view associated to the ant using this
   *   data structure
   *  @throws NullPointerException if <CODE>id</CODE> is null
   */
  public boolean addLast(AntView view, NestId id)
  {
    boolean found = false;
    for (int i=0; i < size && !found; i++)
      found = id.equals(nests[i]);
    nests[size] = id;
    size++;
    return found;
  }
  
  /**
   *  Removes the last element of the path and returns it to
   *  the invoker. Returns null if there are no more elements
   *  in the path.
   *  <BR>
   *  The element is removed by simply reducing variable
   *  size.
   *  
   *  @param view the current view associated to the ant using this
   *   data structure
   */
  public NestId removeLast(AntView view)
  {
    if (size > 0) {
      size--;
      return nests[size];
    } else {
      return null;
    }
  }
  
  /**
   *  Return the current length of the path.
   */
  public int size()
  {
    return size;
  }
  
  /**
   *  Return the current length of the path.
   */
  public int capacity()
  {
    return nests.length;
  }

} // END ReturnPath
