/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.types;

import java.io.*;
import java.net.URL;

/**
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.2 $
 */
public class SimpleURLResource
implements URLResource
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Name associated to this resource */
  private String name;

  /** Url to this resource. */
  private String url;

  /** Document ID associated to this object */
  private DocumentId id;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Constructs a new <CODE>SimpleURLResource</CODE> identified by the
   *  specified <code>URL</code>. 
   *
   *  @param url  url from which the resource may be downloaded
   *  @param id   the document identifier associated to this resource
   */
  public SimpleURLResource(URL url, DocumentId id)
  {
    this.url = url.toString();
    this.name = getFile(this.url);
    this.id = id;
  }

  /**
   *  Constructs a new <CODE>SimpleURLResource</CODE> identified by the
   *  specified stringified URL and by the given document ID.
   *  
   *  @param url url from which the resource may be downloaded
   *  @param id   the document identifier associated to this resource
   */
  public SimpleURLResource(String url, DocumentId id)
  {
    this.url = url;
    this.name = getFile(this.url);
    this.id = id;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the string identifier of the object.
   */
  public String getName()
  {
    return name;
  }


  /**
   *  Not supported.
   */
  public String getAttribute(String name)
  {
    throw new UnsupportedOperationException();
  }


  /**
   *  Return an input stream to download the contents of the
   *  resource. Depending on the implementation, the contents
   *  may be read from the local storage, if the resource has
   *  been copied locally; or downloaded remotely, otherwise.
   *
   *  @return an input stream to read the contents of a resource;
   */
  public InputStream getInputStream()
    throws IOException
  {
    URL Url = new URL(url);
    return Url.openStream();
  }


  /**
   *  Returns the lenght of the content of the resource, if available.
   *  Returns -1 otherwise.
   *
   *  @return the lenght of the content of the resource
   */
  public long getContentLength()
  {
    return (long) id.getLength();
  }

  /**
   *  Returns the space needed to store the resource, if available.
   *  Returns -1 otherwise.
   */
  public long size()
  {
    return 1;
  }

  /**
   *  Returns the document key for this Url resource as the key.
   */
  public Object getKey()
  {
    return id;
  }

  /**
   *  Returns the stringified url for this Url resource.
   */
  public String getURL()
  {
    return url;
  }

  /**
   *  Returns the file part of this URL.
   */
  public String getFile()
  {
    return name;
  }
  
  
  /**
   *  Returns the file part of this URL.
   */
  private String getFile(String url)
  {
    int index = url.lastIndexOf("/");
    return (index != -1) ? url.substring(index+1) : url;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Check whether this <CODE>SimpleURLResource</CODE> is equal to the specified object.
   */
  public boolean equals(Object obj)
  {
    try {
      SimpleURLResource cmp = (SimpleURLResource) obj;
      if (id.equals(cmp.id) && url.equals(cmp.url)) 
        return true;
    } catch (Exception e) { }
    return false;
  }

  /**
   *  Returns a hash code value for the object.
   */
  public int hashCode()
  {
    return id.hashCode();
  }
  
  /**
   *  Returns a string representation of the object.
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("[URLResource: name=");
    buf.append(name);
    buf.append(", id=");
    buf.append(id);
    buf.append(", url=");
    buf.append(url);
    buf.append("]");
    return buf.toString();
  }

} // END SimpleURLResource

