/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.types;

import java.io.*;
import java.net.URL;
import java.util.*;

import anthill.Resource;

/**
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.2 $
 */
public class URLResourceImpl
implements Resource, Externalizable
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** String identifier of the resource. */
  private String name;

  /** Url to this resource. */
  private String url;

  /** Document ID associated to this object */
  private DocumentId id;

  /** Map for attributes */
  private HashMap map;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Default constructor for externalization.
   */
  public URLResourceImpl()
  {
  }

  /**
   *  Constructs a new <CODE>URLResourceImpl</CODE> identified by the
   *  specified <code>URL</code>. 
   *
   *  @param url  url from which the resource may be downloaded
   *  @param id   the document identifier associated to this resource
   */
  public URLResourceImpl(URL url, DocumentId id)
    throws IOException
  {
    this.name = url.getFile();
    this.url = url.toString();
    this.id = id;
    map = null;
  }

  /**
   *  Constructs a new <CODE>URLResourceImpl</CODE> identified by the
   *  specified stringified URL and by the given document ID.
   *  
   *  @param url url from which the resource may be downloaded
   *  @param id   the document identifier associated to this resource
   */
  public URLResourceImpl(String url, DocumentId id)
  {
    this.url = url;
    this.name = getFile(url);
    this.id = id;
    map = null;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the string identifier of the object.
   */
  public String getName()
  {
    return name;
  }


  /**
   *  Returns the value associated to the specified attribute
   *  name, or null if there is no attribute with that name.
   *
   *  @param name the name of the attribute
   */
  public String getAttribute(String name)
  {
    if (map != null)
      return (String) map.get(name);
    else 
      return null;
  }


  /**
   *  Return an input stream to download the contents of the
   *  resource. Depending on the implementation, the contents
   *  may be read from the local storage, if the resource has
   *  been copied locally; or downloaded remotely, otherwise.
   *
   *  @return an input stream to read the contents of a resource;
   */
  public InputStream getInputStream()
    throws IOException
  {
    URL Url = new URL(url);
    return Url.openStream();
  }


  /**
   *  Returns the lenght of the content of the resource, if available.
   *  Returns -1 otherwise.
   *
   *  @return the lenght of the content of the resource
   */
  public long getContentLength()
  {
    return (long) id.getLength();
  }

  /**
   *  Returns the space needed to store the resource, if available.
   *  Returns -1 otherwise.
   */
  public long size()
  {
    return 1;
  }

  /**
   *  Returns the document key for this Url resource as the key.
   */
  public Object getKey()
  {
    return id;
  }

  /**
   *  Returns the stringified url for this Url resource.
   */
  public String getURL()
  {
    return url;
  }

  /**
   *  Returns the file part of this URL.
   */
  public String getFile(String url)
  {
    int index = url.lastIndexOf("/");
    return (index != -1) ? url.substring(index+1) : url;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Check whether this <CODE>URLResourceImpl</CODE> is equal to the specified object.
   */
  public boolean equals(Object obj)
  {
    if (obj != null && obj instanceof URLResourceImpl) {
      URLResourceImpl urlRsrc = (URLResourceImpl) obj;
      if (!id.equals(urlRsrc.id))
        return false;
      if (!name.equals(urlRsrc.name))
        return false;
      if (!url.equals(urlRsrc.url))
        return false;
      return true;
    } else {
      return false;
    }
  }

  /**
   *  Returns a hash code value for the object.
   */
  public int hashCode()
  {
    return id.hashCode();
  }
  
  /**
   *  Returns a string representation of the object.
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("[URLResource: name=");
    buf.append(name);
    buf.append(", id=");
    buf.append(id);
    buf.append(", url=");
    buf.append(url);
    buf.append("]");
    return buf.toString();
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Package methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Insert a mapping between the specified attribute
   *  name and the specified attribute value.
   *
   *  @param name the attribute name
   *  @param value the attribute value
   */
  void setAttribute(String name, String value)
  {
    if (map == null)
      map = new HashMap();
    map.put(name, value);
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Externalizable
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Marshals this <CODE>URLResourceImpl</CODE> to the specified output stream.
   */
  public void readExternal(ObjectInput in)
    throws IOException, ClassNotFoundException
  {
    // Read main fields
    name = in.readUTF();
    url = in.readUTF();
    id = (DocumentId) in.readObject();

    // Read attributes
    int size = in.readInt();
    for (int i=0; i < size; i++) 
      map.put(in.readUTF(), in.readUTF());
  }

    
  /**
  * Marshals this <CODE>URLResourceImpl</CODE> to the specified output stream.
  */
  public void writeExternal(ObjectOutput out)
    throws IOException
  {
    // Write main fields
    out.writeUTF(name);
    out.writeUTF(url);
    out.writeObject(id);

    // Write attributes
    out.writeInt(map.size());
    Iterator i = map.keySet().iterator();
    while (i.hasNext()) {
      String key = (String) i.next();
      out.writeUTF(key);
      out.writeUTF((String) map.get(key));
    }
  }

} // END URLResourceImpl

