/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.util;

import java.io.Serializable;
import java.util.*;

import anthill.types.Closeness;


/**
 *  This class implements a TreeList, which is a combination of a
 *  TreeMap and a doubly linked list. The linked listed is always in
 *  sorted order according to the least recently used (LRU) scheme,
 *  while the TreeMap is sorted according to the keys of the map.
 *
 *  @author Hein Meling
 *  @version $Revision: 1.12 $
 */
public class TreeList
extends MapList implements Serializable
{

  /** Maximum number of values to be returned from the get(key, valcnt) method */
  public static final int MAX_VALUES = 20;

  /**
   *  A proxy for the actual map, used to access methods
   *  from the SortedMap interface.
   */
  protected SortedMap proxyMap;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiate a new TreeList object by setting the maximum number of
   *  key, value pairs which may be contained in it.
   *
   *  @param maxsize  Maximum number of objects which may be contained in
   *    this TreeList. If zero, there is no upper limit to the the size.
   */
  public TreeList(int maxsize, int maxtime)
  {
    super(maxsize, maxtime);
    map = new TreeMap();
    proxyMap = (SortedMap) map;
  }


  /**
   *  Instantiate a new TreeList object by setting the maximum number of
   *  key, value pairs which may be contained in it.
   *
   *  @param maxsize  Maximum number of objects which may be contained in
   *    this TreeList. If zero, there is no upper limit to the the size.
   */
  public TreeList(int maxsize)
  {
    super(maxsize);
    map = new TreeMap();
    proxyMap = (SortedMap) map;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Extended methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns a set of object values whose keys closely match that of
   *  the provided key.  The valuecount specifies how may values close
   *  to the key should be returned.  If not enough keys/values can be
   *  found, a null entry is returned in the remaining positions in the
   *  returned array.
   *
   *  <P> Note that the valuecount is bounded within MAX_VALUES.
   *
   *  @param key  key whose closely associated values is to be returned.
   *    The provided key must implement the <code>Closeness</code>
   *    interface to facilitate the proximity comparision.
   *  @param valuecount  how many values to return.
   *  @return value set matching or closely matching the specified key.
   */
  public synchronized Object[] get(Object key, int valuecount)
  {
    if (!(key instanceof Closeness))
      throw new IllegalArgumentException("Provided object does not implement the Closeness interface");
    Closeness thekey = (Closeness) key;

    if (valuecount > MAX_VALUES)
      valuecount = MAX_VALUES;
    Object[] val = new Object[valuecount];
    SortedMap hMap = proxyMap.headMap(key);
    Iterator tIter = proxyMap.tailMap(key).keySet().iterator();
    Closeness h = null, t = (tIter.hasNext() ? (Closeness) tIter.next() : null);
    for (int i = 0; i < valuecount; i++) {
      if (hMap.isEmpty()) {
        if (t != null) {
          val[i] = ((Entry) proxyMap.get(t)).value;
          t = (tIter.hasNext() ? (Closeness) tIter.next() : null);
        } else {
          break;
        }
      } else {
        h = (Closeness) hMap.lastKey();
        if (t != null) {
          if (thekey.isCloser(h, t) == 1) {
            /*
             * The tail key is closer to the key than the head key.
             */
            val[i] = ((Entry) proxyMap.get(t)).value;
            t = (tIter.hasNext() ? (Closeness) tIter.next() : null);
          } else {
            /*
             * The head key is closer to the key than the tail key (or
             * the key is equally close to both the head and tail keys,
             * in which case we could choose either one; we use the head
             * key).
             */
            val[i] = ((Entry) proxyMap.get(h)).value;
            hMap = hMap.headMap(h);
          }
        } else {
          val[i] = ((Entry) proxyMap.get(h)).value;
          hMap = hMap.headMap(h);
        }
      }
    }
    return val;
  }

} // END TreeList
