/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.freeant;

import java.io.IOException;

import anthill.*;
import anthill.factory.Factory;
import anthill.types.HashedKeyword;
import anthill.util.ExtendedRandom;

/**
 *  Gnutant Pheromone is "the routing table" of the Gnutant species.
 *  Ants may only add knowledge about new keywords, and the nest where
 *  documents relating to that keyword may be found.  The underlying
 *  data structure will take care of limiting the resource (memory) usage,
 *  by implementing an LRU policy.  Ants may also obtain a list of nests
 *  that contain more information about a given keyword or closely matching
 *  keywords.
 *
 *  @author Hein Meling
 *  @version $Revision: 1.6 $
 */
public class GnutantPheromone
implements ResourceManager
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   * Random number generator
   */
  private static ExtendedRandom random = new ExtendedRandom();

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Identifier used in the configuration files */
  public static final String NAME = "GnutantPheromone";
  
  /**
   *  String name of the parameter used to determine 
   */
  public static final String PAR_CAPACITY = "Capacity";
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Probability to follow a random path instead of following the pheromone
   *  trail.
   */
  private float explorationProbability;
  
  /**
   *  Number of attempts to move to another nest, selected randomly among the
   *  nests included in this pheromone, using a normal distribution over the
   *  nests addressing the closest keys.
   */
  private int trailAttempts;
  
  /**
   *  
   */
  private double trailDev;
  
  
  /**
   *  Number of attempts to move to another  nest, selected randomly among the
   *  nests included in this pheromone, using a uniform distribution.
   */
  private int explorationAttempts;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Factory */
  private Factory factory;
  
  /**
   *  Array containing keys maintained in this pheromone. Keys may be duplicated,
   *  in which case are associated to different hosts.
   */
  private HashedKeyword[] keys;
  
  /**
   *  Array containing the identifiers of the nests associated to the keys
   *  contained in the <code>keys</code> array. Nest identifier in position
   *  <code>i</code> corresponds to key in position <code>i</code>.
   */
  private NestId[] nests;
  
  /**
   *  Current index where the next (key, nest id) will be written. Index
   *  ranges from 0 to keys.length-1, wrapping around. In this way, the
   *  tables are used as circular queues.
   */
  private int index;
  
  /**
   *  Number of elements contained in the arrays.
   */
  private int size;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Creates a new <code>GnutantPheromone</code> instance.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public GnutantPheromone(Factory factory, String name)
  {
    this.factory = factory;
    int capacity = factory.intValue(NAME + "." + PAR_CAPACITY);
    keys = new HashedKeyword[capacity];
    nests = new NestId[capacity];
    index = 0;
    size = 0;
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Public methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *
   */
  public void init(AntView view, float explorationProbability,
    int trailAttempts, double trailDev, int explorationAttempts)
  {
    /* Use ant view to initialize the table content */
    Neighbour[] neighbours = view.getNeighbours();
    for (int i=0; i < neighbours.length; i++) {
      HashedKeyword keyhash = (HashedKeyword) factory.get(HashedKeyword.NAME);
      keyhash.init(null);
      addKeyhash(keyhash, neighbours[i].getNestId());
    }
    
    /* Initialize parameters */
    this.explorationProbability = explorationProbability;
    this.trailAttempts = trailAttempts;
    this.trailDev = trailDev;
    this.explorationAttempts = explorationAttempts;
  }
  
  /**
   *
   */
  public void addKeyhash(HashedKeyword keyhash, NestId nestid)
  {
    keys[index] = keyhash;
    nests[index] = nestid;
    index++;
    if (index == keys.length) 
      index = 0;
    if (size < keys.length)
      size++;
  }

  /**
   *
   */
  public boolean moveToNextTrailNest(AntView view, HashedKeyword keyhash)
  {
    boolean exploration = random.tossCoin(explorationProbability);
    
    /* If this ant does not want to explore, first we try to move using
     * the closeness relation; if successful, the method return true. 
     * Otherwise, the ants tries to perform a random step.
     */
    if (!exploration) {
      int[] positions = null;
      double dev = trailDev;
      for (int i=0; i < trailAttempts; i++) {
        int rank = random.nextAbsoluteNormal(1, dev);
        if (positions == null || rank > positions.length)
          positions = getRank(keyhash, rank);
        // System.out.println("Searched: " + keyhash);
        // for (int j=rank-1; j >= 0; j--) 
        //  System.out.println(keys[positions[j]]);
        
        try {
          view.move(nests[positions[rank-1]]);
          return true;
        } catch (IOException e) {
          System.out.println("*");
          // Do nothing; do another attempt
        }
        System.out.println("+");
        /*  We increase the deviation, to obtain greater positions.
         */
        dev = dev*2;
      }
    }
    return moveToNextRandomNest(view);
  }


  /**
   *   The ant performs a random step, either because the ant wants to 
   *   explore, or because the ants failed to move to the selected nests.
   */
  public boolean moveToNextRandomNest(AntView view) 
  {
    for (int i=0; i < explorationAttempts; i++) {
      int pos = random.nextInt(size);
      try {
        view.move(nests[i]);
        return true;
      } catch (IOException e) {
        // Do nothing; do another attempt
      }
    }
    return false;
  }
    

  /**
   *  Search the element with the specified rank in the array.
   *  Complexity of this algorithm, in the worst case, is
   *  keys.length*rank. This is a reasonable complexity when
   *  rank is small. Otherwise, there is an algorithm (listed in
   *  Cormen, Leiserson, Rivest), that works in O(keys.length) time.
   */
  private int[] getRank(HashedKeyword key, int rank)
  {
    if (rank < 1)
      throw new IllegalArgumentException("Rank must be greater than zero");
    if (size == 0)
      throw new IllegalArgumentException("Pheromone table is empty");
    int indexes[] = new int[rank];
    indexes[0] = 0;
    int found = 1;
    
    for (int i=1; i < size; i++) {
      boolean added = false;
      if (found < rank) {
        indexes[found] = i;
        found++;
        added = true;
      } else if (key.isCloser(keys[i], keys[indexes[found-1]]) < 0) {
        indexes[found-1] = i;
        added = true;
      }
      if (added) {
        /*  The current last element is greater than the maximum value maintained
         *  in the distances array; so, we can safely remove this maximum
         *  value and substitute it with the current element
         */
        int j= found-1;
        
        /*  At this point, the last element may be smaller than the other
         *  elements; we move it to the correct position, by performing
         *  exchanges between elements.
         */
        while (j > 0 && key.isCloser(keys[indexes[j]],keys[indexes[j-1]]) < 0) {
          int temp = indexes[j-1];
          indexes[j-1] = indexes[j];
          indexes[j] = temp;
          j--;
        }
      }
    }
    if (keys[indexes[rank-1]].getKeyword() != null)
      System.out.print("A");
    else
      System.out.print(".");
    return indexes;
  }
        
        
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Returns a string representation of the <code>GnutantPheromone</code>.
   *
   *  @return  a <code>String</code> representation of the GnutantPheromone
   *    object. A multiline string will be returned.
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("[------ Pheromone table ------------\n");
    for (int i=0; i < size; i++) {
      buf.append(keys[i]);
      buf.append("  -->  ");
      buf.append(nests[i]);
      buf.append("\n");
    }
    buf.append("-----------------------------------]\n");
    return buf.toString();
  }

} // END GnutantPheromone


