/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.freeant;

import java.util.*;

import anthill.factory.Factory;
import anthill.nest.Nest;
import antsim.*;

/**
 *  This class implements a generic request generator. To create a 
 *  request, the generator selects randomly (with uniform probability) 
 *  a resource from a resource set, creates a generic request for that 
 *  resource and assign it randomly to one of the nests included in the 
 *  peernet. The number of requests that are generated for each invocation
 *  of method <CODE>generateRequests</CODE> can be configured through
 *  the <CODE>NumberOfRequests</CODE> parameter.
 * 
 *  @author Alberto Montresor
 *  @author Hein Meling
 *  @version $Revision: 1.4 $
 */
public class GnutantRequestGenerator
implements RequestGenerator
{
 
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Random generator */
  private static Random random = new Random();

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   *  If the Selection parameter assumes this value, resources 
   *  are selected randomly (with uniform distribution) from the 
   *  resource set.
   */
  public static final String UNIFORM = "Uniform";
  
  /** 
   *  If the Selection parameter assumes this value, resources 
   *  are selected randomly (with exponential distribution) from 
   *  the resource set.
   */
  public static final String EXPONENTIAL = "Exponential";

  /** 
   *  
   */
  private static final int RANDOM_UNIFORM = 0;
  
  /** 
   *  
   */
  private static final int RANDOM_EXPONENTIAL = 1;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Parameter name used to read the number of requests to be generated */
  public static final String PAR_NUMBERREQUESTS = "NumberOfRequests";
  
  /** 
   *  String name of the parameter used to determine the type of
   *  selection you can use. The possible values are: "Sequntial",
   *  "Uniform", "Exponential"
   */
  public static final String PAR_SELECTION = "Selection";

  /** 
   *  String name of the parameter used to determine vary the mean of
   *  the keyword selection distribution.
   */
  public static final String PAR_MEAN = "Mean";
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   *  Number of requests to be generated whenever method
   *  <CODE>generateRequests</CODE> is invoked.
   */
  private int nrequests;  
  
  /**
   *  The kind of distribution to be used when selecting documents
   *  to be placed in the system.
   */
  private int selection;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Factory object used to read configuration parameters and classes */
  private Factory factory;

  /** List of nests on which to generate requests */
  private List nests;

  /** Set of resources from which to select a request to be generated */
  private List resources;

  /** The response monitor object for this request generator */
  private ResponseMonitor monitor;

  /** Can be pre-computed: log(beta) */
  private double logbeta;

  /** Can be pre-computed: pow(beta, n-1) = beta^(n-1) */
  private double powbeta;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Constructs a new generic request generator and reads parameter
   *  <CODE>NumberOfRequests</CODE> from the configuration files.
   */
  public GnutantRequestGenerator(Factory factory, String name)
  {
    this.factory = factory;
    clear();
  }

  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from RequestGenerator interface
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void init(ResourceSet resourceSet, Peernet peernet, ResponseMonitor monitor)
  {
    nests = peernet.getNests();
    resources = resourceSet.getResources();
    this.monitor = monitor;

    nrequests = factory.intValue(NAME + "." + PAR_NUMBERREQUESTS);
    String sel = factory.stringValue(NAME + "." + PAR_SELECTION);
    if (sel.equals(UNIFORM))
      selection = RANDOM_UNIFORM;
    else if (sel.equals(EXPONENTIAL))
      selection = RANDOM_EXPONENTIAL;

    /* Mean value for the keyword selection distribution (geometric) */
    if (selection == RANDOM_EXPONENTIAL) {
      /*
       * Details about these computations can be found in the file
       * various/SIMULATION.
       */
      double beta = 1.0 - (1.0/factory.doubleValue(NAME + "." + PAR_MEAN));
      logbeta = Math.log(beta);
      powbeta = Math.pow(beta, resources.size() - 1);
    }
  }

  /**
   *  Generates a set of requests in the specified P2P network.
   *  Different implementations may adopt different generation and
   *  distribution patterns.
   */
  public void generateRequests()
  {
    if (selection == RANDOM_UNIFORM) {

      /* Choose random nest and make a request */
      for (int j=0; j < nrequests; j++) {
        int nestIndex = random.nextInt(nests.size());
        int resourceIndex = random.nextInt(resources.size());
        SearchRequest request = 
          new SearchRequest( (String) resources.get(resourceIndex) );
        Nest nest = (Nest) nests.get(nestIndex);
        Rid rid = new Rid(nest);
        monitor.startRequest(rid);
        nest.request(rid, request, monitor);
      }

    } else {

      /* Choose random nest and make a request */
      for (int j=0; j < nrequests; j++) {
        int nestIndex = random.nextInt(nests.size());
        int resourceIndex = computeIndex();
        SearchRequest request = 
          new SearchRequest( (String) resources.get(resourceIndex) );
        Nest nest = (Nest) nests.get(nestIndex);
        Rid rid = new Rid(nest);
        monitor.startRequest(rid);
        nest.request(rid, request, monitor);
      }

    }
  }

  /**
   *  Details about this function can be found in the file
   *  various/SIMULATION.
   */
  private int computeIndex()
  {
    double x = random.nextDouble();
    double key = Math.log(x + powbeta * (1-x)) / logbeta;
    return (int) Math.round(key);
  }

} // END RequestGenerator
