/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.freeant;

import java.util.*;

import anthill.Resource;
import anthill.factory.Factory;
import anthill.storage.*;

/**
 *  Implementation of an Least Recently Used (LRU) based URL storage.
 *
 *  @author Hein Meling
 *  @version $Revision: 1.4 $
 */
public class UrlStorage
implements QueryStorage
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** 
   *  The identifier of the configuration parameter 
   */
  public static final String PAR_CAPACITY = "Capacity";
  
  /** 
   *  The identifier of the configuration parameter 
   */
  public static final String PAR_CONTROLLERNAME = "ControllerName";

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Factory used to instantiate objects */
  private Factory factory;

  /** Name used to identify this component in the factory */
  protected String name;

  /** The query controller provided by the implementing class. */
  private QueryController queryCtrl;

  /**
   *  Array containing the keys currently in the storage. The array
   *  is organized as a circular queue, in order to implement a LRU policy.
   */
  private Object[] keys;
  
  /** 
   *  Array containing the resources currently in the storage. The array
   *  is organized as a circular queue, in order to implement a LRU policy.
   */
  private Resource[] resources;
  
  /**
   *  Current index in the circular table. 
   */
  private int index;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public UrlStorage(Factory factory, String name)
  {
    this.factory = factory;
    this.name = name;
    clear();
  }

  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
    /* Instantiates the configured controller */
    String controllerName = factory.stringValue(name + "." + PAR_CONTROLLERNAME);
    queryCtrl = (QueryController) factory.get(controllerName);

    /* Instantiates the storage tables, reading the configured capacity */
    int capacity = factory.intValue(name + "." + PAR_CAPACITY);
    keys = new Object[capacity];
    resources = new Resource[capacity];
    
    /* Initializes the writing index */
    index = 0;
  }
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns true if the resource associated to <CODE>key</CODE> is contained in
   *  the storage, false otherwise.
   *
   *  @param key  the key to be searched
   *  @return true if the resource associated to <CODE>key</CODE> is contained in
   *    the storage
   */
  public boolean containsResource(Object key)
  {
    for (int i=0; i < resources.length; i++) {
      if (key.equals(keys[i])) {
        return true;
      }
    }
    return false;
  }
  
  /**
   *  Returns the resource associated with the specified key. If the key
   *  is not associated with any resource, a null value is returned.
   *
   *  @param key  whose associated resource to retreive.
   *  @return resource with which the specified key is associated.
   */
  public Resource getResource(Object key)
  {
    for (int i=0; i < resources.length; i++) {
      if (key.equals(keys[i])) {
        return resources[i];
      }
    }
    return null;
  }

  /**
   *  Adds a resource to the storage and associates it to the specified
   *  key. If the remaining capacity of the storage is not sufficient to
   *  store the resource, a <CODE>CapacityException</CODE> is thrown
   *  containing information about the remaining capacity. Method
   *  <CODE>size()</CODE> may be invoked on the resource
   *  to determine the amount of space required by the resource. 
   *  In order to be stored on persistent storage, resource must 
   *  implement the <CODE>Serializable</CODE> interface.
   *
   *  This is a specialized implementation that will add URLs pointing
   *  to a file with the same <CODE>resourceKey</CODE> to a URL resource
   *  set before adding that URL set to the storage.
   *
   *  @param resourceKey  the key identifying the URL resource
   *  @param urlResource  the URL resource to be added
   *  @throws CapacityException  raised if the given resource object
   *    exceeds the total available capacity for this storage.
   */
  public void addResource(Object key, Resource resource)
    throws CapacityException
  {
    /** Search if the URL resource is already in the storage */
    for (int i=0; i < resources.length; i++) {
      if (resource.equals(resources[i])) {
        /*  If the resource is already present in position i, we move it
         *  to the front; the element to be replaced is moved in position i. 
         */
        resources[i] = resources[index];
        keys[i] = keys[index];
        break;
      }
    }
    resources[index] = resource;
    keys[index] = key;
    index++;
    if (index == resources.length)
      index = 0;
  }

  /**
   *  
   */
  public Resource[] getResources(Object query) 
  {
    ArrayList list = new ArrayList();
    for (int i=0; i < resources.length; i++) {
      if (resources[i] != null && queryCtrl.satisfies(resources[i], query))
          list.add(resources[i]);
    }
    Resource[] res = new Resource[list.size()];
    list.toArray(res);
    return res;
  }

  /**
   *  Removes the resource associated with the specified key
   *  from the storage.
   *
   *  @param key the identifier of the resource to be removed
   */
  public void removeResource(Object key)
  {
    for (int i=0; i < resources.length; i++) {
      if (key.equals(keys[i])) {
        resources[i] = null;
        keys[i] = null;
        // FIXME: Effectively save space
      }
    }
  }
  
  /**
   *  Returns the complete set of resources stored in this storage object
   *  using an iterator.
   *
   *  @return iterator representing this storage object, allowing to
   *    inspect its content.
   */
  public Iterator iterator()
  {
    throw new UnsupportedOperationException();
  }
  
  
  /**
   *  Returns the initial capacity of this generic storage.
   */
  public long getCapacity()
  {
    return keys.length;
  }
  
  
  /**
   *  Returns the remaining capacity of this generic storage.
   */
  public long getRemainingCapacity()
  {
    return keys.length;
  }
  
} // END UrlStorage

