/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.gnutant;

import java.util.*;

import anthill.*;
import anthill.factory.Factory;
import anthill.types.HashedKeyword;
import anthill.util.Log;

/**
 *
 *  @author Alberto Montresor
 *  @author Hein Meling
 *  @version $Revision: 1.3 $
 */
public class GnutantFactory
implements AntFactory
{
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** The tokens used to split a keyword string. */
  private static final String TOKENS = " _-.";

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Factory reference */
  private Factory factory;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Factory for creating <i>Gnutant ant species</i>.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public GnutantFactory(Factory factory, String name)
  {
    this.factory = factory;
  }

  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from the AntFactory interface
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *   Returns a set of ants for a query. Each ant will take care of
   *   the research of a distinct token of the query string. 
   */
  public Ant[] getAnts(RequestId rid, Object request, NestId requestor)
  {
    InsantRequest ir = null;
    String query = null;
    HashedKeyword keyhash[];

    if (request instanceof InsantRequest) {
      ir = (InsantRequest) request;
      keyhash = computeKeyhashes(ir.keywords);
    } else if (request instanceof String) {
      query = (String) request;
      keyhash = computeKeyhashes(query);
    } else {
      Log.println(600, "RequestObject: " + request);
      throw new IllegalArgumentException("Request object not recongnized.");
    }
    /* We create a list of ants that will take care of this request */
    ArrayList list = new ArrayList();

    /* Create an ant for each of the keywords in the request */
    for (int i = 0; i < keyhash.length; i++) {
      Ant ant;
      if (ir != null)
        ant = (Ant) new Insant(keyhash[i], ir.urlResource);
      else
         ant = (Ant) new SearchGnutant(rid, query, keyhash[i]);
         //ant = (Ant) new Gnutant(rid, query, keyhash[i], requestor);
      list.add(ant);
    }
    // Convert the ant list to an array
    Ant[] ants = (Ant[]) list.toArray(new Ant[list.size()]);
    return ants;
  }
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Private methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Compute keyword hashes from the specified string.
   *
   *  @param keywordString  a keyword string for which to compute hashes.
   *  @return  an array of <code>HashedKeyword</code> values corresponding to
   *    the specified keywords.
   */
  private HashedKeyword[] computeKeyhashes(String keywordString)
  {
    /* Fragment the keywordString into keywords, separated by
     * spaces, underscore, dash and dot. */
    StringTokenizer tokenizer = new StringTokenizer(keywordString, TOKENS);
    HashedKeyword keyhash[] = new HashedKeyword[tokenizer.countTokens()];
    for (int i = 0; tokenizer.hasMoreTokens(); i++) {
      /* Obtain keyword hashes from the keywordString. */
      keyhash[i] = (HashedKeyword) factory.get(HashedKeyword.NAME);
      keyhash[i].init(tokenizer.nextToken());
    }
    return keyhash;
  }

} // END GnutantFactory


