/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.gnutant;

import java.util.*;

import anthill.*;
import anthill.factory.Factory;
import anthill.types.HashedKeyword;
import anthill.util.TreeList;

/**
 *  Gnutant Pheromone is "the routing table" of the Gnutant species.
 *  Ants may only add knowledge about new keywords, and the nest where
 *  documents relating to that keyword may be found.  The underlying
 *  data structure will take care of limiting the resource (memory) usage,
 *  by implementing an LRU policy.  Ants may also obtain a list of nests
 *  that contain more information about a given keyword or closely matching
 *  keywords.
 *
 *  @author Hein Meling
 *  @version $Revision: 1.6 $
 */
public class GnutantPheromone
implements ResourceManager
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Identifier used in the configuration files */
  public static final String NAME = "GnutantPheromone";

  /**
   *  String name of the parameter used to determine the number of
   *  repetitions to be executed.
   */
  public static final String PAR_CAPACITY = "Capacity";
  
  /**
   *  String name of the parameter used to determine the number of
   *  iterations to be executed.
   */
  public static final String PAR_CLOSENESS = "Closeness";
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Closeness defines the number of close keys to retreive from
   *  the <code>pheroMap</code>.
   */
  private int closeness;

  /**
   *  Capacity defines the maximum size of the gnutant pheromone.
   */
  private int capacity;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Factory reference, used to create hashed keywords */
  private Factory factory;

  /**
   *  A TreeList associating keyword hash codes and NestIds.
   *  TreeList implements a LRU policy, mean that keywords
   *  that have been stored may be removed based on the
   *  least recently used scheme. This is to make room for
   *  new entries in the index. The underlying map is a
   *  TreeMap, thus ensuring that keys (key hash codes) are
   *  in stored in sorted order.
   */
  private TreeList pheroMap;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Creates a new <code>GnutantPheromone</code> instance. The object is
   *  created through the factory; possible initialization is performed 
   *  through <CODE>init</CODE> methods.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public GnutantPheromone(Factory factory, String name)
  {
    this.factory = factory;
    capacity = factory.intValue(NAME + "." + PAR_CAPACITY);
    closeness = factory.intValue(NAME + "." + PAR_CLOSENESS);
  }

  /**
   *  Initialization method that uses the information in the capacity 
   *  
   *  @param view an <code>AntView</code> value
   */
  void init(AntView view)
  {
    pheroMap = new TreeList(capacity);
    Neighbour[] neighbours = view.getNeighbours();
    for (int i=0; i < neighbours.length; i++) {
      HashedKeyword keyhash = (HashedKeyword) factory.get(HashedKeyword.NAME);
      keyhash.init(null);
      addKeyhash(keyhash, neighbours[i].getNestId());
    }
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Public methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Add a new nest containing a document that match the specified
   *  keyword, represented by a hash of the keyword.
   *  <P>
   *  Note that this method is synchronized because there may be a race
   *  condition if trying to simultaneously add the same keyhash with
   *  different nest identifiers.
   *
   *  @param keyhash  Hash of the keyword represented by a
   *    <CODE>HashedKeyword</CODE> object.
   *  @param nestid   Identifier for the nest holding a document
   *    represented by the specified keyword.
   *  @return  <CODE>true</CODE> if the given keyword is already known.
   *    That is, other nests store information about this keyword.
   *    <CODE>false</CODE> is returned if the keyword is unknown.
   */
  public synchronized boolean addKeyhash(HashedKeyword keyhash, NestId nestid)
  {
    //Log.println("Adding " + keyhash + " " + nestid);
    if (pheroMap.containsKey(keyhash)) {
      /* Keyword already exists, add this nest to this entry. */
      ((Set) pheroMap.get(keyhash)).add(nestid);
//       Log.print("+");
      return true;
    } else {
      /* Keyword not previously stored, create new entry in the pheroMap. */
      Set nestSet = new HashSet(5);
      nestSet.add(nestid);
      pheroMap.put(keyhash, nestSet);
      return false;
    }
  }

  /**
   *  Obtains a set of possible next nests based on the provided
   *  <code>keyhash</code>.  If there is no exact match for the
   *  specified keyhash in the underlying datastructure the closes
   *  match will be made available in the first position in the
   *  returned ArrayList.
   *
   *  @param keyhash  the keyword to match against to find the set
   *    of possible next nests.
   *  @param path   a list of nests to avoid, since we have already
   *    visited them.
   *  @return a list of possible next nests, sorted according to their
   *    closeness to the specified keyhash.
   */
  public ArrayList getNextNest(HashedKeyword keyhash, ArrayList path)
  {
    // We allocate some space to avoid doing this when calling addAll() later.
    ArrayList nxtList = new ArrayList(10);
    Set nests = null;
    /*
     * Obtain a set of nests satisfying the closeness relation, sorted
     * according to their closeness to keyhash.  The first element of
     * the array may be equal to the keyhash.
     */
    Object[] closeValues = pheroMap.get(keyhash, closeness);
    for (int i = 0; i < closeValues.length; i++) {
      if (closeValues[i] != null) {
        nests = (Set) closeValues[i];
        /* Remove already visited nests. */
        if (path != null)
          nests.removeAll(path);

        /* Remove nests already in the nxtList to avoid duplicate entries. */
        nests.removeAll(nxtList);

        /* Add remaining nests to the end of the nxtList. */
        nxtList.addAll(nests);
      } else {
        /* Exit the for-loop, no more values. */
        break;
      }
    } // end of for (int i = 0; i < closeValues.length; i++)

    /*
     * Returns an array list of nests in sorted order of
     * their closeness to the given keyhash. 
     */
    return nxtList;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns a string representation of the <code>GnutantPheromone</code>.
   *  Since this a sort routing table, it will typically be large, so using
   *  this for outputing to the screen is ill-advised.  It is mainly useful
   *  for logging the content of the pheromone tables at various times during
   *  a run.
   *
   *  @return  a <code>String</code> representation of the GnutantPheromone
   *    object. A multiline string will be returned.
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("  [----\n  " + this.getClass().getName() + ": \n");
    /* Get the set of keys in LRU order */
    ArrayList keys = pheroMap.keyList();
    for (int i = 0; i < keys.size(); i++) {
      HashedKeyword keyhash = (HashedKeyword) keys.get(i);
      buf.append("  " + keyhash.getKeyword() + ":  ");
      Iterator j = ((Set) pheroMap.get(keyhash)).iterator();
      while (j.hasNext())
        buf.append(j.next() + ", ");
      buf.append("\n");
    }
    buf.append("  ----]");
    return buf.toString();
  }

} // END GnutantPheromone
