/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.jxtagnutant;

import java.io.*;
import java.util.ArrayList;

import anthill.*;
import anthill.storage.*;
import anthill.types.HashedKeyword;

/**
 *
 *  @author Hein Meling
 *  @author Francesco Russo (frusso@cs.unibo.it)
 *  @version $Revision: 1.8 $
 */
public class BackwardGnutant
implements Ant, Serializable
{
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  private static java.util.Random rand;
	static 
  {
	rand = new java.util.Random();
  }

  /** The provider of these resources; originator of this ant */  
  private NestId provider;

  /** The hash value of the keyword searched by this ant */
  private HashedKeyword keyhash;

  /** Resources that we wish to put update the distributed index with */
  private Resource[] resources;

  /** 
   *  Variable path contains the list of visited nodes, allowing the ant
   *  to avoid visiting the same nests multiple times.
   */
  private ArrayList path;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Default constructor for externalization
   */
  public BackwardGnutant()
  {
	System.out.println("--- BackwardGnutant: created!");
  }

  /**
   *  Initializes a new BackwardGnutant with the set of resources to
   *  be used for updating the distributed index.
   */
  public BackwardGnutant(HashedKeyword keyhash, Resource[] resources, ArrayList path)
  {
  	/*
   	*  We remove the provider; since we don't want to move to the
   	*  originator of this ant.
   	*/
  	
	this.provider = (NestId) path.remove(path.size()-1);
  	this.keyhash = keyhash;
  	this.resources = resources;
  	this.path = path;
  	//System.out.println("Going back with " + resources.length + " resources!");
  	System.out.println("--- BackwardGnutant: created!");
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  // Documentation inherited by interface
  public String getSpecies()
  {
 	return "jxtagnutant";
  }

  public void init(AntView view)
  {
  }
  
  /**
   *  This method encapsulate the computation performed by the ant.
   *
   *  @param view reference to the AntView object which
   *    constitutes the controlled environment of the ant
   */
  public void run(AntView view)
  {
  	/*
   	*  Updates the routing information for the keyhash it is searching.
   	*/
  	System.out.println("--- BackwardGnutant: run() - getting pheromone"); 
  	//GnutantPheromone pher = (GnutantPheromone) view.getPheromone();
	GnutantPheromone pher = (GnutantPheromone) view.getResourceManager( Storage.PHR_STORAGE );
  	
  	if (pher != null)
	    System.out.println("--- BackwardGnutant: run() - pheromone is not null"); 
  	if ( ! provider.toString().trim().toLowerCase().equals(view.getNestId().toString().trim().toLowerCase()) )
	    System.out.println("--- BackwardGnutant: run() - the 2 ids are different");
  
  	if (pher != null && !( provider.toString().trim().toLowerCase().equals(view.getNestId().toString().trim().toLowerCase()) ) )
  	{
	    /* Add the keyhash and the provider nest to the local pheromone table. */
	    pher.addKeyhash(keyhash, provider);
	    System.out.println("--- BackwardGnutant: run() - added new info to the pheromone"); 
	    ((jxtaimpl.AntView)view).updatePheromone( pher );
		//view.createResourceManager( Storage.PHR_STORAGE );
	    System.out.println("--- BackwardGnutant: run() - saved it all!");
  	}
  
  	/*
   	*  Add resources to the system 
   	*/
  	System.out.println("--- BackwardGnutant: run() - getting URL_STORAGE"); 
  	ResourceManager storage = null;
  	try
  	{
	    //storage = (QueryStorage) view.getStorage(Storage.URL_STORAGE);
		storage = (ResourceManager) view.getResourceManager( Storage.URL_STORAGE );
  	}
  	catch(StorageNotFoundException snfe)
  	{
	    storage = null;
  	}
  	if (storage == null)
  	{
	    //throw new IllegalStateException("URL Storage not found");
	    System.out.println("--- BackwardGnutant: run() - URL_STORAGE is null");
  	}
  	else
  	{   
	    System.out.println("--- BackwardGnutant: run() - URL_STORAGE is NOT null!");
	    for (int i=0; i < resources.length; i++) 
	    {
	        try 
	        {
			    QueryStorage qStorage = ((QueryStorage)storage);
		        qStorage.addResource(resources[i].getKey(), resources[i]);
		    } 
	  		catch (CapacityException e) 
			{
				System.out.println("Size: " + resources[i].size());
		 		System.err.println(e);
			}
		}
  	}
  	/*
   	*  Move to the predecessor of the current nest by removing the last entry
   	*  in the path for each move. This will shrink the path as we backtrack.
   	*/
  	int pathsize = path.size();
  	if (pathsize > 0) 
  	{
	    try 
	    { 
	    	System.out.println("--- BackwardGnutant: run() - about to move");
		  	view.move((NestId) path.remove(pathsize-1)); 
			System.out.println("--- BackwardGnutant: run() - done!");
			return;
		} 
		catch (IOException ioe) 
		{ 
	  		//* if something goes wrong with the selected path entry let's try
	  		//* the following one out
	   		move( view );
	 		return;
		} 
  	}
  	/*
   	* We have reached the originating nest; we commit suicide. 
   	*/
  }

  /**
  * Since the <code>BackwardGnutant</code> is in charge of updating the distributed
  * index walking backward it may happen that a nest is no more reachable. In such
  * a scenario what we do is just jumping to the next nest in our reversed path hoping
  * it is still alive.
  */
  private void move(AntView view)
	{
	int pathsize = path.size();
	 
	try
	{
	  if (pathsize > 0)
	  {
		System.out.println("--- BackwardGnutant: move() - about to move");
		view.move((NestId) path.remove(pathsize-1)); 
		System.out.println("--- BackwardGnutant: move() - done!");
	  }
	  else
		System.out.println("--- BackwardGnutant: move() - NO MORE ENTRIES IN THE PATH!");
	}
	catch(IOException ioe)
	{
	  move( view );
	}
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Externalization methods
  ////////////////////////////////////////////////////////////////////////////////////////////


} // END BackwardGnutant

