/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.jxtagnutant;

import java.io.*;
import java.util.*;

import anthill.util.Util;

/**
 *  Provides a holder for keywords, read from an ASCII file with a set of
 *  keywords on each line.
 *
 *  @author Hein Meling
 *  @version $Revision: 1.2 $
 */
public class Keywords
{
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Random number generator */
  private static Random random = new Random();

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** List of keywords to select from. */
  private ArrayList keywordList;

  /** The log value of beta; used for the distribution function. */
  private double logBeta;

  /** The power function: beta^(n+1) */
  private double powBeta;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Loads the specified keyword file, entirely.
   *
   *  @param kwFile  the filename of the keyword file.
   */
  public Keywords(String kwFile)
  throws FileNotFoundException
  {
  /* We assume that the file does not contain more than MAX_VALUE lines. */
  this(kwFile, Integer.MAX_VALUE, 200);
  }


  /**
   *  Loads the specified keyword file, limited by the number of keyword
   *  lines to load into the keyword list.
   *
   *  @param kwFile  the filename of the keyword file.
   *  @param lines   the number of lines to load; used to limit the
   *    number of keyword lines to load into the <code>keywordList</code>.
   */
  public Keywords(String kwFile, int lines, double mean)
  throws FileNotFoundException
  {
  if (mean <= 14) 
    throw new IllegalArgumentException("Mean value for the geometric distribution must be greater than 13.");

  BufferedReader br = new BufferedReader(new FileReader(kwFile));
  keywordList = new ArrayList(lines);
  try {
    String keyline = null;
    /* Get the next line of keywords from the keyword file */
    while (((keyline = br.readLine()) != null) && (keywordList.size() < lines))
    keywordList.add(keyline);
  } catch (IOException e) {
    /* Serious failure */
    Util.panic(e);
  }
  double beta = 1.0 - (1.0/mean);
  logBeta = Math.log(beta);
  powBeta = Math.pow(beta, keywordList.size() + 1); // n + 1
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Public Access methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the keyword stored in the given position.  If the given
   *  index position is outside the table range, it will wrap to a legal
   *  position using modulo table size.
   *
   *  @param index  position into the keywordList.
   *  @return a <code>String</code> containing a set of keywords
   *   associated with the given index position in the keywordList.
   */
  public String getKeyword(int index)
  {
  return (String) keywordList.get(index % keywordList.size());
  }

  /**
   *  Returns a random (uniformly distributed) keyword from the
   *  keywordList.
   *
   *  @return a <code>String</code> containing a set of keywords.
   */
  public String getRndKeyword()
  {
  return getKeyword(random.nextInt(keywordList.size()));
  }

  /**
   *  Returns a random (exponentional distribution) keyword from the
   *  keywordList.
   *
   *  @return a <code>String</code> containing a set of keywords.
   */
  public String getLogRndKeyword()
  {
  return getKeyword(compute());
  }

  /**
   * gnuplot:
   *   m=14  (Lowest legal value)
   *   b=1.0-(1.0/m)
   *   plot [x=0:1] log(x+b**(1+n)*(1-x))/log(b)
   */
  private int compute()
  {
  double x = random.nextDouble();
  double key = Math.log(x + powBeta * (1-x)) / logBeta;
  return (int) Math.round(key);
  }

} // END Keywords
