/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import java.util.*;

import anthill.*;
import anthill.factory.Factory;
import anthill.util.ExtendedRandom;

/*
 *  History: a first version, based only on arrays and using binary
 *  search, is included in the repository (ants.messor.ArrayMessorPheromone,1.1)
 */

/**
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.2 $
 */
public class ArrayMessorPheromone
implements MessorPheromone
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Random number generator
   */
  private static ExtendedRandom random = new ExtendedRandom();
  
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Parameter name used to read PheromoneCapacity */
  public static final String PAR_CAPACITY = "Capacity";
  
  /** Parameter name used to read TimeRange */
  public static final String PAR_TIMERANGE = "TimeRange";
  
  /** Parameter name used to read max visits */
  public static final String PAR_MAXVISITS = "MaxVisits";

  /** Parameter name used to read visits */
  public static final String PAR_MINVISITS = "MinVisits";

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Capacity of the pheromone table */
  int capacity;
  
  /** Time range */
  int timerange;
  
  /** Maximum number of visits that can occur in the last <CODE>timerange</CODE> rounds */
  int maxVisits;

  /** Minimum number of visits that can occur in the last <CODE>timerange</CODE> rounds */
  int minVisits;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Factory reference used to read configuration parameters */
  Factory factory;
  
  /** This data structures maps ids to entries */
  Map idMap;
    
  /** Table to perform uniform selection */
  PheromoneEntry[] table;

  /* Comparator for ordering */
  Comparator comparator;
  
  /** 
   * Virtual timer used to distinguish among the age of the entries;
   * it is updated whenever a new group of entries is inserted in the
   * pheromone table.
   */
  int time;
  
  /** 
   *  Contains the number of visiting ants in the last rounds, computed 
   *  per round.
   */
  int[] visits;
  
  /**
   *  Contains the total number of visiting ants in the last rounds.
   */
  int totalVisits;
  
  /**
   *  Next position where to write when visiting a node.
   */
  int index;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Creates a new ArrayMessorPheromone, with the specified capacity.
   * Three sorted sets/maps of entries are created, in order to 
   * keep entries ordered by ascending loads, descending loads
   * and by lru arrival time. Furthermore, a linear array of
   * entries is also maintained, to enable to randomly select
   * a nest in constant time.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public ArrayMessorPheromone(Factory factory, String name)
  {
    this.factory = factory;
    clear();
  }
  
  /**
   * Restores the object to its initial state.
   */
  public void clear()
  {
    /* Read initialization parameters */
    capacity =  factory.intValue(NAME + "." + PAR_CAPACITY);
    timerange = factory.intValue(NAME + "." + PAR_TIMERANGE);
    maxVisits = factory.intValue(NAME + "." + PAR_MAXVISITS);
    minVisits = factory.intValue(NAME + "." + PAR_MINVISITS);
    
    /* Creates comparators */
    Comparator comparator = new LoadComparator();
    
    /* Creates an empty map*/
    idMap = new HashMap();
    
    /* Creates a new table for random selection */
    table = new PheromoneEntry[capacity];
    
    /* Initialize virtual timer */
    time = 0;
    
    /** Initialize visits counters */
    visits = new int[timerange];
    totalVisits = 0;
    /*  Insert "pseudo visits" in the counter, to avoid creation of multiple ants 
     *  at the beginning
     */
    for (int i=0; i < timerange; i++) {
      visits[i] = (minVisits-totalVisits) / (timerange-i);
      totalVisits += visits[i];
    }
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods 
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   * 
   */
  public void addNest(NestId id, int load)
  {
    /* Add 1 to the timer associated to this node */
    time++;
    
    // Log.println("addNest: " + id + ", " + load);
    PheromoneEntry entry = (PheromoneEntry) idMap.get(id);
    if (entry == null) {

      /*
       * New nest identifier; creates a new entry.
       */
      entry = new PheromoneEntry(id, load, time);
      if (idMap.size() < capacity) {
        
        /* Still space in the pheromone table; the index in the array
         * is obtained by simply taking the next empty element */
        entry.index = idMap.size();
        
      } else {
        
        /* Capacity exhausted; deletes the oldest entry in the pheromone,
         * by removing it from the various sets and maps, and re-assigning
         * its index in the array */
        int imin = 0;
        for (int i=0; i < capacity; i++) {
          if (table[i].time < table[imin].time)
            imin = i;
        }
        entry.index = imin;
        idMap.remove(table[imin].id);
      }
      
      /* Insert the element in the data structure */
      idMap.put(id, entry);
      table[entry.index] = entry;

    } else {
      
      /* Nest identifier already present; update it */
      entry.load = load;
      entry.time = time;

    }
    
  }

    
  /**
   *  Returns the next nest to be followed, depending on
   *  some parameters and on the content of the storage.
   */
  public NestId getNextMaxNest(double deviation)
  {
    if (idMap.size() == 0) {
      throw new IllegalStateException("Pheromone table is empty");
    }

    /* Sort table in ascending order */
    Arrays.sort(table, 0, idMap.size());
    
    /* If there is no difference between max and min, it is better to go randomly */
    int lmin = table[0].load;
    int lmax = table[idMap.size()-1].load;
    //System.out.println(emin.id + "," + emin.load + " - " + emax.id + "," + emax.load);
    if (lmin == lmax) {
      return getRandomNest();
    }
    
    int index = 0;
    do {
      index = Math.abs(random.nextNormal((int) 0, deviation));
    } while (index >= idMap.size());
    return table[idMap.size()-1-index].id;
  }
  
  /**
   * Returns the next nest to be followed, depending on
   * some parameters and on the content of the storage.
   */
  public NestId getNextMinNest(double deviation)
  {
    if (idMap.size() == 0) {
      throw new IllegalStateException("Pheromone table is empty");
    }
    
    /* Sort table in ascending order */
    Arrays.sort(table, 0, idMap.size());

    /* If there is no difference between max and min, it is better to go randomly */
    int lmin = table[0].load;
    int lmax = table[idMap.size()-1].load;
    //System.out.println(emin.id + "," + emin.load + " - " + emax.id + "," + emax.load);
    if (lmin == lmax) {
      return getRandomNest();
    }
    
    int index = 0;
    do {
      index = Math.abs(random.nextNormal((int) 0, deviation));
    } while (index >= idMap.size());
    return table[index].id;
  }

  /**
   * 
   */
  public NestId getRandomNest()
  {
    if (idMap.size() == 0) {
      throw new IllegalStateException("Pheromone table is empty");
    }
    int index = random.nextInt(idMap.size());
    return table[index].id;
  }
  
  
  /**
   * 
   */
  public void update(NestId local, NestId[] nests, int[] loads)
  {
    /* Add the visited nests to the system */
    for (int i=0; i < nests.length; i++) {
      if (nests[i] != null && !local.equals(nests[i]))
        addNest(nests[i], loads[i]);
    }
  }
    
  /**
   * 
   */
  public void update(NestId local, Neighbour[] nests)
  {
    if (idMap.size() == 0) {
      /* Add the visited nests to the system */
      for (int i=0; i < nests.length; i++) {
        if (!local.equals(nests[i].getNestId()))
          addNest(nests[i].getNestId(), 0);
      }
    }
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Visit management methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  If the number of visits is smaller than the minimum amount of visits to be
   *  received, returns the number of missing visits. Return 0 otherwise. 
   */
  public boolean isUnderloaded()
  {
    totalVisits -= visits[index];
    index = (index + 1) % timerange;
    int diff = minVisits - totalVisits;
    return (diff > 0);
  }
  
  /**
   *  Returns true if the node is overloaded by ants, that have to be removed
   *  from the nest.
   */
  public boolean isOverladed()
  {
    if (totalVisits >= maxVisits) 
      return true;
    totalVisits++;
    visits[index]++;
    return false;
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Statistics methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Add the nest identifiers contained in the nest to
   *  the set identified
   */
  public PheromoneEntry[] getEntryTable()
  {
    return table;
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Inner classes 
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   * 
   */
  class LoadComparator
    implements Comparator
  {
    /**
     * Constructs a new load comparator.
     */
    LoadComparator()
    {
    }
    
    /**
     * Compares two object. The results depends on 
     * the order (ascending or descending).
     */
    public int compare(Object o1, Object o2)
    {
      //Log.println("loadComparator: o1=" + o1 + ", o2=" + o2);
      PheromoneEntry e1 = (PheromoneEntry) o1;
      PheromoneEntry e2 = (PheromoneEntry) o2;
      int res;
      if (e1.load < e2.load) {
        res = -1;
      } else if (e1.load == e2.load) {
        // FIXME Currently, only LocalNid are comparable;
        Comparable obj = (Comparable) e1.id;
        return obj.compareTo(e2.id); 
      } 
      return 1;
    }
    
  } // END LoadComparator
  
} // END ArrayMessorPheromone

