/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import java.util.List;

import anthill.*;
import anthill.factory.Factory;

/**
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.7 $
 */
public class MessorAntFactory
implements AntFactory
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Parameter name used to read maxExploringProb */
  public static final String MAX_EXPLORING_PROB = "MaxExploringProb";
  
  /** Parameter name used to read minExploringProb */
  public static final String MIN_EXPLORING_PROB = "MinExploringProb";
  
  /** Parameter name used to read maxSearchDev */
  public static final String MAX_SEARCH_DEV = "MaxSearchDev";
  
  /** Parameter name used to read minSearchDev */
  public static final String MIN_SEARCH_DEV = "MinSearchDev";

  /** Parameter name used to read maxKeepSearchingProb */
  public static final String MAX_KEEPSEARCHING_PROB = "MaxKeepSearchingProb";
  
  /** Parameter name used to read minKeepSearchingProb */
  public static final String MIN_KEEPSEARCHING_PROB = "MinKeepSearchingProb";
  
  /** Parameter name used to read maxTargetRatio */
  public static final String MAX_TARGET_RATIO = "MaxTargetRatio";
  
  /** Parameter name used to read maxTargetRatio */
  public static final String MIN_TARGET_RATIO = "MinTargetRatio";
  
  /** Parameter name used to read transportPercentage */
  public static final String TRANSPORT_PERCENTAGE = "TransportPercentage";
  
  /** Parameter name used to read transportPercentage */
  public static final String MAX_TRANSPORT = "MaxTransport";
  
  /** Parameter name used to read VisitedBufferSize */
  public static final String VISITED_BUFFER_SIZE = "VisitedBufferSize";
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** 
   *  When in MaxSearching status, probability to explore the network 
   *  instead to go towards nodes with max load.
   */
  private float maxExploringProb;
  
  /** 
   *  When in MinSearching status, probability to explore the network 
   *  instead to go towards nodes with min load.
   */
  private float minExploringProb;
  
  /** 
   *  When in MaxSearching status, standard deviation for selecting
   *  the next node using a normal distribution.
   */
  private double maxSearchDev;
  
  /** 
   *  When in MinSearching status, standard deviation for selecting
   *  the next node using a normal distribution.
   */
  private double minSearchDev;
  
  /**
   *  When in MaxSearching status, probability to keep searching when
   *  a node with a load greater than the given treshold has been 
   *  reached. 
   */
  private float maxKeepSearchingProb;
  
  /**
   *  When in MaxSearching status, probability to keep searching when
   *  a node with a load greater than the given treshold has been 
   *  reached. 
   */
  private float minKeepSearchingProb;
  
  /**
   *  Target ratio between the load of overloaded and underloaded 
   *  nests when in SearchMax status. When an ant is looking for 
   *  overloaded machines, it keeps moving until the ratio between 
   *  the load observed in the nest with maximum load and the nest 
   *  with minimum load do not reach this target value.
   */
  private float maxTargetRatio;
  
  /**
   *  Target ratio between the load of overloaded and underloaded 
   *  nests when in SearchMin status. When an ant is looking for 
   *  underloaded machines, it keeps moving until the ratio between 
   *  the load observed in the nest with maximum load and the nest 
   *  with minimum load do not reach this target value.
   */
  private float minTargetRatio;
  
  /**
   *  Percentage of tasks to be move from one site to another.  
   */
  private float transportPercentage;
  
  /**
   *  Max number of tasks that can be transported by one node to another  
   */
  private int maxTransport;

  /**
   *  Size of the buffer containing the visited nests  
   */
  private int visitedBufferSize;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Factory object used to read parameters from configuration files */
  Factory factory;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public MessorAntFactory(Factory factory, String name)
  {
    this.factory = factory;
    clear();
  }
  
  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
    maxExploringProb = 
      factory.floatValue(NAME + "." + MAX_EXPLORING_PROB);
    minExploringProb = 
      factory.floatValue(NAME + "." + MIN_EXPLORING_PROB);
    maxSearchDev = 
      factory.floatValue(NAME + "." + MAX_SEARCH_DEV);
    minSearchDev = 
      factory.floatValue(NAME + "." + MIN_SEARCH_DEV);
    maxKeepSearchingProb = 
      factory.floatValue(NAME + "." + MAX_KEEPSEARCHING_PROB);
    minKeepSearchingProb = 
      factory.floatValue(NAME + "." + MIN_KEEPSEARCHING_PROB);
    maxTargetRatio = 
      factory.floatValue(NAME + "." + MAX_TARGET_RATIO);
    minTargetRatio = 
      factory.floatValue(NAME + "." + MIN_TARGET_RATIO);
    transportPercentage = 
      factory.floatValue(NAME + "." + TRANSPORT_PERCENTAGE);
    maxTransport = 
      factory.intValue(NAME + "." + MAX_TRANSPORT);
    visitedBufferSize = 
      factory.intValue(NAME + "." + VISITED_BUFFER_SIZE);
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *   Returns a set of ants for a query. Each ant will take care of
   *   the research of a distinct token of the query string. 
   */
  public Ant[] getAnts(RequestId rid, Object request, NestId requestor)
  {
    Ant[] ants = new Ant[1];
    if (request instanceof List) {
      /* Use a deliver ant to local deliver the messages */
      ants[0] = new DeliverAnt((List) request);
    } else {
      ants[0] = new MessorAnt(
        maxExploringProb, minExploringProb, maxSearchDev, minSearchDev, 
        maxKeepSearchingProb, minKeepSearchingProb, maxTargetRatio, minTargetRatio, 
        transportPercentage, maxTransport, visitedBufferSize);
    }
    return ants;
  }
  
} // END MessorAntFactory


