/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import java.io.Serializable;

import anthill.*;


/**
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.3 $
 */
public class MessorQueen
implements Ant, Serializable
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   *  When in MaxSearching status, probability to explore the network 
   *  instead to go towards nodes with max load.
   */
  private float maxExploringProb;
  
  /** 
   *  When in MinSearching status, probability to explore the network 
   *  instead to go towards nodes with min load.
   */
  private float minExploringProb;
  
  /** 
   *  When in MaxSearching status, standard deviation for selecting
   *  the next node using a normal distribution.
   */
  private double maxSearchDev;
  
  /** 
   *  When in MinSearching status, standard deviation for selecting
   *  the next node using a normal distribution.
   */
  private double minSearchDev;

  
  /**
   *  When in MaxSearching status, probability to keep searching when
   *  a node with a load greater than the given treshold has been 
   *  reached. 
   */
  private float maxKeepSearchingProb;
  
  /**
   *  When in MaxSearching status, probability to keep searching when
   *  a node with a load greater than the given treshold has been 
   *  reached. 
   */
  private float minKeepSearchingProb;
  
  /**
   *  Target ratio between the load of overloaded and underloaded 
   *  nests when in SearchMax status. When an ant is looking for 
   *  overloaded machines, it keeps moving until the ratio between 
   *  the load observed in the nest with maximum load and the nest 
   *  with minimum load do not reach this target value.
   */
  private float maxTargetRatio;
  
  /**
   *  Target ratio between the load of overloaded and underloaded 
   *  nests when in SearchMin status. When an ant is looking for 
   *  underloaded machines, it keeps moving until the ratio between 
   *  the load observed in the nest with maximum load and the nest 
   *  with minimum load do not reach this target value.
   */
  private float minTargetRatio;
  
  /**
   *  Percentage of tasks to be move from one site to another.  
   */
  private float transportPercentage;
  
  
  /**
   *  
   */
  private int visitedBufferSize;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * 
   */
  public MessorQueen(
    float maxExploringProb, float minExploringProb,
    double maxSearchDev, double minSearchDev,
    float maxKeepSearchingProb, float minKeepSearchingProb,
    float maxTargetRatio, float minTargetRatio,
    float transportPercentage, int visitedBufferSize)
  {
    /* Stores parameters */
    this.maxExploringProb = maxExploringProb;
    this.minExploringProb = minExploringProb;
    this.maxSearchDev = maxSearchDev;
    this.minSearchDev = minSearchDev;
    this.maxKeepSearchingProb = maxKeepSearchingProb;
    this.minKeepSearchingProb = minKeepSearchingProb;
    this.maxTargetRatio = maxTargetRatio;
    this.minTargetRatio = minTargetRatio;
    this.transportPercentage = transportPercentage;
    this.visitedBufferSize = visitedBufferSize;
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Ant
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Returns the identifier of the "species" of this ant.
   *  Different ant species may have access to different
   *  pheromone information in nests.
   */
  public String getSpecies()
  {
    return "MessorAnt";
  }

  public void init(AntView view)
  {
  }

  /**
   *  This method encapsulate the computation performed by the
   *  ant. 
   *
   *  @param view reference to the AntView object which
   *    constitutes the controlled environment of the ant
   */
  public void run(AntView view)
  {
    /* Obtain storage classes */
    MessorPheromone pheromone = 
      (MessorPheromone) view.createResourceManager(MessorPheromone.NAME);

    /* Add itself to be scheduled again */
    view.addAnt(this);
    
    /* Add missing ants */
    if (pheromone.isUnderloaded()) {
      MessorAnt ant = new MessorAnt(
        maxExploringProb, minExploringProb, maxSearchDev, minSearchDev, 
        maxKeepSearchingProb, minKeepSearchingProb, maxTargetRatio, minTargetRatio, 
        transportPercentage, 0, visitedBufferSize); // FIXME!!!!!!!!!
      view.addAnt(ant);
    }
  }
  
} // END MessorQueen

