/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import java.util.*;

import anthill.factory.Factory;
import anthill.nest.Nest;
import anthill.util.ExtendedRandom;
import antsim.*;

/**
 *  This implementation generates fake computing request to be simulated
 *  in the Anthill environment. 
 * 
 *  
 *  @author Alberto Montresor
 *  @version $Revision: 1.7 $
 */
public class MessorRequestGenerator
implements RequestGenerator
{
 
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Random number generator */
  static ExtendedRandom random = new ExtendedRandom();

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  String name of the parameter used to 
   */
  public static final String PAR_MEANTBE = "MeanTicksBetweenEvents";

  /**
   *  String name of the parameter used to 
   */
  public static final String PAR_DEVTBE = "DevTicksBetweenEvents";

  /**
   *  String name of the parameter used to 
   */
  public static final String PAR_MEANPPE = "MeanPeersPerEvent";
  
  /**
   *  String name of the parameter used to 
   */
  public static final String PAR_DEVPPE = "DevPeersPerEvent";
  
  /**
   *  String name of the parameter used to 
   */
  public static final String PAR_MEANRPP = "MeanRequestsPerPeer";
  
  /**
   *  String name of the parameter used to 
   */
  public static final String PAR_DEVRPP = "DevRequestsPerPeer";
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Mean number of ticks between two request events (Ticks Between 
   *  Events, TBE). See <CODE>anthill.util.ExtendedRandom</CODE> 
   *  for an explanation of mean parameters.
   * 
   *  @see anthill.util.ExtendedRandom
   */
  private int meanTBE;
  
  /**
   *  Standard deviation for the number of ticks between two request
   *  events (Ticks Between Events, TBE). See 
   *  <CODE>anthill.util.ExtendedRandom</CODE> for an explanation 
   *  of deviation parameters.
   * 
   *  @see anthill.util.ExtendedRandom
   */
  private double devTBE;
  
  
  /**
   *  The mean number of peers in which requests are generated in a single
   *  generation events (Peer Per Event, PPE). See 
   *  <CODE>anthill.util.ExtendedRandom</CODE> for an explanation 
   *  of mean parameters.
   * 
   *  @see anthill.util.ExtendedRandom
   */
  private int meanPPE;
  
  /**
   *  Standard deviation for the number of peers in which requests are 
   *  generated in a single generation events (Peer Per Event, PPE). 
   *  See <CODE>anthill.util.ExtendedRandom</CODE> for an explanation 
   *  of deviation parameters.
   * 
   *  @see anthill.util.ExtendedRandom
   */
  private double devPPE;

  /**
   *  The mean number of requests generated in each
   *  peer during a generation event (Request Per Peer, RPP). See 
   *  <CODE>anthill.util.ExtendedRandom</CODE> for an explanation 
   *  of mean parameters.
   * 
   *  @see anthill.util.ExtendedRandom
   */
  private int meanRPP;
  
  /**
   *  Standard deviation for the number of requests generated in each
   *  peer during a generation event (Request Per Peer, RPP).
   *  See <CODE>anthill.util.ExtendedRandom</CODE> for an explanation 
   *  of deviation parameters.
   * 
   *  @see anthill.util.ExtendedRandom
   */
  private double devRPP;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Factory object used to read configuration parameters and classes */
  private Factory factory;

  /** Ticks to go */
  private int ticks;

  /** List of peer nodes (nests) on which to generate requests */
  private List peers;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   * 
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public MessorRequestGenerator(Factory factory, String name)
  {
    this.factory = factory;
    clear(); 
  }
  
  /**
   *  Restores the object to its initial state. 
   */
  public void clear()
  {
    /* Read configuration parameters */
    meanTBE = factory.intValue(NAME + "." +PAR_MEANTBE);
    devTBE = factory.doubleValue(NAME + "." +PAR_DEVTBE);
    meanPPE = factory.intValue(NAME + "." +PAR_MEANPPE);
    devPPE = factory.doubleValue(NAME + "." +PAR_DEVPPE);
    meanRPP = factory.intValue(NAME + "." +PAR_MEANRPP);
    devRPP = factory.doubleValue(NAME + "." +PAR_DEVRPP);
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  public void init(ResourceSet resourceSet, Peernet peernet, ResponseMonitor monitor)
  {
    peers = peernet.getNests();
  }


  /**
   *  In this simulator, this method is invoked at each tick. 
   */
  public void generateRequests()
  {
    /* In this current version, */ 
    if (ticks <= 0) {
      /* Generate requests */
      int npeers = random.nextNormal(meanPPE, devPPE);
      for (int i=0; i < npeers; i++) {
        int index = random.nextInt(peers.size());
        Nest nest = (Nest) peers.get(index);
        
        int nrequests = random.nextNormal(meanRPP, devRPP);
        ArrayList tasks = new ArrayList(nrequests);
        for (int j=0; j < nrequests; j++) 
          tasks.add(new SimulatedTask());
        nest.request(new Rid(nest), tasks, null);
      }
      ticks = random.nextNormal(meanTBE, devTBE);
    }
    ticks--;
  }

  
} // END RequestGenerator

