/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.impl;

import anthill.factory.Factory;
import anthill.util.Analyzer;
import antsim.*;

/**
 *  This class contains a simple implementatin of the Experiment
 *  interface, that can be used to quickly test the behavior of
 *  an ant species by repeating the scenario a certain number of
 *  times. When evaluated, returns an array of statistics objects;
 *  each element of the array contains the statistics related to
 *  one repetition.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.4 $
 */
public class ExperimentImpl
implements Experiment
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Property identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  String name of the Java property used to determine whether the
   *  the analyisis of the footprint is activated.
   */
  public static final String PROP_FOOTPRINT = "Footprint";
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  String name of the parameter used to determine the number of
   *  repetitions to be executed.
   */
  public static final String PAR_REPETITIONS = "Repetitions";

  /**
   *  String name of the parameter used to determine the number of
   *  iterations to be executed.
   */
  public static final String PAR_ITERATIONS = "Iterations";
 
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** 
   *  Number of iterations composing an experiment. During a single 
   *  iteration, method <CODE>evaluate</CODE> of the scenario associated
   *  to this experiment is evaluated.
   */
  private int iterations;
  
  /** 
   *  Number of repetions of this experiment. The experiment is
   *  repeated a certain number of times, in order to obtain mean
   *  values.
   */
  protected int repetitions;

  /** 
   *  If true, methods for the analysis of experiment footprint are 
   *  invoked 
   */
  protected boolean footprint;
  
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Factory object used to read configuration parameters and classes */
  protected Factory factory;
  
  /** Scenario on which evaluate the experiment */
  protected Scenario scenario;
 
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates a new time experiment. Configuration parameters are
   *  read through the specified factory object.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public ExperimentImpl(Factory factory, String name)
  {
    this.factory = factory;
    clear(); 
    
    /* Read system properties */
    footprint = Boolean.getBoolean(PROP_FOOTPRINT);
  }
  
  /**
   *  Restores the object to its initial state. 
   */
  public void clear()
  {
    /* Read configuration parameters */
    iterations = factory.intValue(NAME + "." +PAR_ITERATIONS);
    repetitions = factory.intValue(NAME + "." +PAR_REPETITIONS);
    
    /* Creates local variables */
    if (scenario == null)
      scenario = (Scenario) factory.get(Scenario.NAME);
    else
      scenario.clear();
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Experiment
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Perform the evaluation of the current ant algorithm. Returns a
   *  <CODE>Stats</CODE> array containing the statistics collected during the
   *  simulation. 
   *  <p>
   *  In this implementation, a single <CODE>Stats</CODE> object is created
   *  and is used to collect statistics for all scenario executions
   *  performed by this <CODE>Experiment</CODE>.
   */
  public Object evaluate()
  {
    long time, elapsed;

    /*  
     *  Create the statistics object needed to collect
     *  statistics about the number of repetitions
     *  of the simulation.
     */
    Stats results[] = new Stats[repetitions];
    for (int i=0; i < repetitions; i++) {
      results[i] = new Stats();
    }

    for (int j=0; j < repetitions; j++) {
      
      results[j].add("Repetition", j+1);
      results[j].add("Iterations", 0);

      /* Compute time for statistics */
      time = System.currentTimeMillis();
      
      for (int i=0; i < iterations; i++) {
        
        results[j].inc("Iterations");
  
        /* Updates statistics and evaluate the scenario */
        scenario.setStats(results[j]);
        scenario.evaluate();
        
      }

      /* Compute elapsed time */
      elapsed = (System.currentTimeMillis()-time);
      time = System.currentTimeMillis();
      results[j].add("Simulation time", elapsed);
      
      /* For debug purposes */
      if (footprint) {
        // Log.println(Analyzer.printClasses(this));
        results[j].add("Footprint", Analyzer.footprint(this));
        elapsed = (System.currentTimeMillis()-time);
        results[j].add("Analyzer time", elapsed);
      }

      /* Print statistics */
      scenario.analyze(results[j]);
      
      scenario.clear();
    }
    return results;
  }

} // END ExperimentImpl

