/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.impl;

import java.util.*;

import anthill.Resource;
import anthill.factory.Factory;
import anthill.nest.Nest;
import anthill.storage.*;
import antsim.*;

/**
 *  Classes implementing this interface are used to generate pre-defined
 *  resources at the beginning of a simulation.
 *  
 *  @author Alberto Montresor
 *  @version $Revision: 1.3 $
 */
public class GenericResourceDistribution
implements ResourceDistribution
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Random generator */
  private static Random random = new Random();

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** 
   *  Parameter name used to read  the number of copies of the resources to be inserted
   *  in the simulated peer-to-peer network.
   */
  public static final String PAR_NUMBERCOPIES = "NumberOfCopies";

  /** 
   *  Parameter name used to read the name of storage in which resources have to be
   *  inserted.
   */
  public static final String PAR_STORAGENAME = "StorageName";
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** 
   *  Number of copies of the resources to be inserted in the simulated
   *  peer-to-peer network.
   */
  private int ncopies;  

  /** Name of the storage in which resource have to be inserted */
  private String storageName;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Factory object used to read configuration parameters and classes */
  private Factory factory;
 
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   * 
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public GenericResourceDistribution(Factory factory, String name)
  {
    this.factory = factory;
    clear();
  }
  
  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
    ncopies = factory.intValue(NAME + "." + PAR_NUMBERCOPIES);
    storageName = factory.stringValue(NAME + "." + PAR_STORAGENAME);
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Distributes resources in a P2P network.
   */
  public void distributeResources(ResourceSet resourceSet, Peernet peernet)
  {
    List resources = resourceSet.getResources();
    List nests = peernet.getNests();
    
    for (int i=0; i < resources.size(); i++) {
      for (int j=0; j < ncopies; j++) {
        int pos = random.nextInt(nests.size());
        Nest nest = (Nest) nests.get(pos);
        GenericStorage storage = 
          (GenericStorage) nest.getResourceManager(storageName, null);
        try {
          Resource resource = (Resource) resources.get(i);
          storage.addResource(resource.getKey(), resource);
        } catch(CapacityException e) {
          System.err.println("Capacity exceeded in storage " + storageName + " located ad nest " + nest.getNestId());
          e.printStackTrace();
        } catch (NullPointerException e) {
          System.err.println("Storage " + storageName + " does not exist in nest " + nest.getNestId());
          e.printStackTrace();
        }
      }
    }
    
  }

} // END GenericResourceDistribution

