/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.impl;

import java.util.*;

import anthill.*;
import anthill.factory.Factory;
import anthill.nest.ResponseListener;
import anthill.util.Clearable;
import antsim.*;


/**
 *  Class <CODE>GenericResponseMonitor</CODE> is 
 *  
 *  @author Alberto Montresor
 *  @author Hein Meling
 *  @version $Revision: 1.3 $
 */
public class GenericResponseMonitor
implements Clearable, ResponseMonitor, ResponseListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Stats */
  private Stats stats;

  /** Responses */
  private HashMap responses;

  /** Time */
  private long time;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor / reinitialization methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Instantiates a new <CODE>GenericResponseMonitor</CODE>.
   *
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public GenericResponseMonitor(Factory factory, String name)
  {
    clear();
  }
  
  
  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
    if (responses == null) {
      responses = new HashMap();
    } else {
      responses.clear();
    }
    time = 0;
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from ResponseMonitor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  
   */
  public void setStatistics(Stats stats)
  {
    this.stats = stats;
    stats.add("TotalRequests", 0);
    stats.add("SatisfiedRequests", 0);
    stats.add("TotalResponses", 0);
    stats.add("LocalFound", 0);
    stats.add("TotalTime", 0);
  }

  /**
   *  
   */
  public void tick()
  {
    time++;
  }

  /**
   *  
   */
  public void tick(int v)
  {
    time += v;
  }

  /**
   *  Must be invoked prior to invoking a request on a nest, to ensure
   *  that the given request is stored to enable lookup on responses.
   */
  public void startRequest(RequestId rid)
  {
    if (responses.containsKey(rid))
      throw new IllegalArgumentException("Duplicated request: " + rid);
    responses.put(rid, new ResponseSet(time));
    stats.inc("TotalRequests");
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from ResponseListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Upcall method used to notify this listener that the given request
   *  has returned the specified set of resources.
   *
   *  @param rid the request identifier for the returned resources.
   *  @param resources the resources found.
   *  @return true  if the timeout has passed.
   */
  public void returnResources(RequestId rid, Resource[] resources)
  {
    returnResources(rid, resources, 0);  
  }


  /**
   *  Upcall method used to notify this listener that the given request
   *  has finished returning resources.
   */
  public void complete()
  {
    // FIXME Review this API
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Protected methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Upcall method used to notify this listener that the request 
   *  associated to it has returned the specified set of resources.
   *
   *  @param rid the request identifier for the returned resources.
   *  @param resources the resources found.
   *  @param timeout 
   *  @return true  if the timeout has passed.
   */
  private boolean returnResources(RequestId rid, Resource[] resources, int timeout)
  {
    ResponseSet set = (ResponseSet) responses.get(rid);
    if (set != null) { 
      long diff = time - set.time;
      if (set.empty) {
        stats.inc("SatisfiedRequests");
        if (diff == 0)
          stats.inc("LocalFound");
        stats.add("TotalTime", diff);
        set.empty = false;
      }
      for (int i = 0; i < resources.length; i++) {
        if (set.responses.add(resources[i])) {
          /* The resource was not present; increment number of unique
           * resources found. */
          stats.inc("TotalResponses");
        }
      }
      if (diff >= timeout) 
        return true;
    }
    return false;
  }


  /**
   *  Returns the responses associated with the specified response identifier.
   *
   *  @param rid the request identifier whose responses to return.
   *  @return the set of responses associated with the specified response identifier.
   */
  private Set responses(RequestId rid)
  {
    ResponseSet set = (ResponseSet) responses.get(rid);
    if (set == null)
      throw new IllegalArgumentException("Unknown request: " + rid);
    return set.responses;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Inner class 
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Response collection
   */
  private class ResponseSet
  {
    /** Request generation time */
    long time;
    
    /** Unique responses */
    Set responses;

    /** True if we have not found any resource for this request */
    boolean empty;
    
    /** Constructor */
    ResponseSet(long time)
    {
      this.time = time;
      responses = new HashSet();
      empty = true;
    }
  } // END ResponseSet


} // END GenericResponseMonitor
  
