/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.impl;

import anthill.factory.Factory;
import anthill.util.Analyzer;
import antsim.*;

/**
 *  This class contains an implementation of experiment in which 
 *  one of the simulation parameter, specified trough parameter
 *  <CODE>ParameterName</CODE>, has its value changed from a 
 *  minimum value (parameter <CODE>MinValue</CODE>) to a maximum 
 *  value (parameter <CODE>MaxValue</CODE>), with an increase
 *  specified by parameter <CODE>Step</CODE>. For each value of the
 *  variable parameter, the simulation is composed of a certain
 *  number of execution method <CODE>evaluate()</CODE> of a scenario;
 *  this number is specified by parameter <CODE>Iterations</CODE>.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.4 $
 */
public class RangeExperiment
implements Experiment
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Property identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  String name of the Java property used to determine whether the
   *  the analyisis of the footprint is activated.
   */
  public static final String PROP_FOOTPRINT = "Footprint";

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   *  String name of the parameter used to identify the name of the
   *  parameter whose value is ranged from a minimum value to a maximum
   *  value.
   */
  public static final String PAR_PARAMETERNAME = "ParameterName";
  
  /**
   *  String name of the parameter used to determine the minimum value
   *  that has to be assigned to the ranged parameter.
   */
  public static final String PAR_MINVALUE = "MinValue";
    
  /**
   *  String name of the parameter used to determine the maximum value
   *  that has to be assigned to the ranged parameter.
   */
  public static final String PAR_MAXVALUE = "MaxValue";
    
  /**
   *  String name of the parameter used to determine the step between
   *  two consecutive values to be assigned to the ranged parameter.
   */
  public static final String PAR_STEP = "Step";

  /**
   *  String name of the parameter used to determine the number of
   *  iterations to be executed.
   */
  public static final String PAR_ITERATIONS = "Iterations";
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Identifier of the parameter to be set */
  protected String parameterName;  

  /* Maximum value */
  protected int maxvalue;  
  
  /* Minimum TTL value */
  protected int minvalue;  

  /* Value used to increment step */
  protected int step; 

  /** Number of times a scenario is evaluated  */
  protected int iterations;
  
  /** If true, methods for the analysis of experiment footprint are invoked */
  protected boolean footprint;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Factory object used to read configuration parameters and classes */
  private Factory factory;
  
  /** Scenario on which evaluate the experiment */
  protected Scenario scenario;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates a new <CODE>RangeExperiment</CODE>. Configuration
   *  parameters are read through the specified factory object.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public RangeExperiment(Factory factory, String name)
  {
    this.factory = factory;
    clear();
    
    /* Read system properties */
    footprint = Boolean.getBoolean(PROP_FOOTPRINT);
  }
   
  /**
   *  Restores the object to its initial state. 
   */
  public void clear()
  {
    /* Read parameters */
    parameterName = factory.stringValue(NAME + "." +PAR_PARAMETERNAME);
    maxvalue = factory.intValue(NAME + "." +PAR_MAXVALUE);
    minvalue = factory.intValue(NAME + "." +PAR_MINVALUE);
    step = factory.intValue(NAME + "." +PAR_STEP);
    iterations = factory.intValue(NAME + "." +PAR_ITERATIONS);
    
    /* Creates local variables */
    if (scenario == null)
      scenario = (Scenario) factory.get(Scenario.NAME);
    else
      scenario.clear();
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Experiment
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Perform the evaluation of the current ant algorithm. Returns a
   *  <CODE>Stats</CODE> array containing the statistics collected during the
   *  simulation. 
   *  <p>
   *  In this implementation, a single <CODE>Stats</CODE> object is created
   *  and is used to collect statistics for all scenario executions
   *  performed by this <CODE>Experiment</CODE>.
   */
  public Object evaluate()
  {
    long time, elapsed;
    
    /* Allocate space for the statistics object to be returned */
    int steps = (maxvalue-minvalue)/step +1;
    Stats[] results = new Stats[steps];

    /* Creates the scenario */
    factory.putValue(parameterName, minvalue);
    scenario = (Scenario) factory.get(Scenario.NAME);

    int value = minvalue;
    for (int i=0; i < steps; i++) {
      
      /* Update value in the factory; the new value will be set when calling clear */
      factory.putValue(parameterName, value);
      
      /* Compute time for statistics */
      time = System.currentTimeMillis();
      
      /* 
       *  Clear the scenario; create a new statistics object; evaluate the scenario 
       */
      results[i] = new Stats();     
      results[i].add(parameterName, value);
      if (iterations > 1)
        results[i].add("Iterations", iterations);
      for (int j=0; j < iterations; j++) {
        System.out.println("Iteration: " + j);
        scenario.clear(); 
        scenario.setStats(results[i]);
        scenario.evaluate();
      }
      
      /* Compute elapsed time */
      elapsed = (System.currentTimeMillis()-time);
      time = System.currentTimeMillis();
      results[i].add("Simulation time", elapsed);

      /* For debug purposes */
      if (footprint) {
        results[i].add("Footprint", Analyzer.footprint(this));
        elapsed = (System.currentTimeMillis()-time);
        results[i].add("Analyzer time", elapsed);
      }
      
      /* Print statistics */
      scenario.analyze(results[i]);
      
      /* Increase value */
      value += step;
      
    }
    return results;
  }

} // END RangeExperiment

