/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.impl;

import java.util.Random;

import anthill.factory.Factory;
import antsim.*;


/**
 *  Class <CODE>ScenarioImpl</CODE> is a simple implementation of
 *  interface <CODE>Scenario</CODE> in which a predefined set of
 *  resources is inserted in the network and then a certain number
 *  of requests are performed and simulate. Initial resources are
 *  generated and distributed in the network using a 
 *  <CODE>ResourceGenerator</CODE> object; at runtime, requests
 *  are generated using a <CODE>RequestGenerator</CODE> object.
 *  
 *  @author Alberto Montresor
 *  @author Hein Meling
 *  @version $Revision: 1.7 $
 */
public class ScenarioImpl
implements Scenario
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Random number generator */
  private static Random random = new Random();
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   * String name of the parameter used to determine 
   */
  public static final String PAR_INITROUNDS = "InitRounds";
  
  /** 
   * String name of the parameter used to determine 
   */
  public static final String PAR_REGULARROUNDS = "RegularRounds";
  
  /** 
   * String name of the parameter used to determine 
   */
  public static final String PAR_FINALROUNDS = "FinalRounds";

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   * Initialization rounds to be performed; during this phase, requests are 
   * generated and statistics are not computed 
   */
  protected int initRounds;

  /** 
   * Regular rounds to be performed; during this phase, requests are 
   * generated and statistics are computed
   */
  protected int regularRounds;

  /** 
   * Final rounds to be performed; during this phase, requests are not
   * generated and statistics are computed
   */
  protected int finalRounds;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Factory used to instantiate objects */
  protected Factory factory;
  
  /** Stats */
  protected Stats stats;

  /** Peernet on which the genetic code will be evaluated */
  protected Peernet peernet;
  
  /** Set containing the initial resources */
  protected ResourceSet resourceSet;
  
  /** Resource distribution used to distribute initial resources */
  protected ResourceDistribution resourceDistribution;
  
  /** Request generator used to generate run-time requests */
  protected RequestGenerator requestGenerator;
  
  /** Response monitor to evaluate response time */
  protected ResponseMonitor monitor;
  
  /** 
   *  Scenario analyzer used to display information about the behavior of
   *  the ant algorithm.
   */
  protected ScenarioAnalyzer analyzer;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor / reinitialization methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates a new ScenarioImpl.
   *
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public ScenarioImpl(Factory factory, String name)
  {
    /* Initializes variables */
    this.factory = factory;
    
    /* Invokes clear*/
    clear();
  }

  
  /**
   * Restores the object to its initial state.
   */
  public void clear()
  {
    /* Read configuration parameters */
    initRounds = factory.intValue(NAME + "." + PAR_INITROUNDS);
    regularRounds = factory.intValue(NAME + "." + PAR_REGULARROUNDS);
    finalRounds = factory.intValue(NAME + "." + PAR_FINALROUNDS);

    /* 
     * Creates a new peernet, if not present; 
     * otherwise clean existing one 
     */
    if (peernet == null) {
      peernet = (Peernet) factory.get(Peernet.NAME);
    } else {
      peernet.clear();
    }

    /* 
     * Creates a new resources set, if not present; 
     * otherwise clean existing one. 
     */
    if (resourceSet == null) {
      resourceSet = (ResourceSet) factory.get(ResourceSet.NAME);
    } else {
      resourceSet.clear();
    }

    /* 
     * Creates a new request generator, if not present; 
     * otherwise clean existing one. Uses the resource generator
     * to generate and distribute pre-defined resourcs.
     */
    if (resourceDistribution == null) {
      resourceDistribution = 
        (ResourceDistribution) factory.get(ResourceDistribution.NAME);
    } else {
      resourceDistribution.clear();
    }
    resourceDistribution.distributeResources(resourceSet, peernet);

    /*
     * Create a new response monitor, if not present; otherwise clean
     * the existing one.
     */
    if (monitor ==  null) {
      monitor = (ResponseMonitor) factory.get(ResponseMonitor.NAME);
    } else {
      monitor.clear();
    }
    
    /* 
     * Creates a new request generator, if not present; 
     * otherwise clean existing one. The resource set is used
     * to initialize the request generator.
     */
    if (requestGenerator == null) {
      requestGenerator = (RequestGenerator) factory.get(RequestGenerator.NAME);
    } else {
      requestGenerator.clear();
    }
    requestGenerator.init(resourceSet, peernet, monitor);

    /* Creates local variables */
    if (analyzer == null) {
      analyzer = (ScenarioAnalyzer) factory.get(ScenarioAnalyzer.NAME);
    } else {
      /* Restore state of existing scenario */
      analyzer.clear();
    }
  }
   
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Scenario
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Set the statistics object which will be used throughout
   * the scenario evaluation.
   */
  public void setStats(Stats stats)
  {
    this.stats = stats;
    monitor.setStatistics(stats);
  }
  

  /**
   * Perform the evaluation of this scenario object by simulating it.
   * The simulation is subdivided in rounds. At each iteration,
   * a new request is generated and assigned to one of the nest
   * included in the simulated peer network, which generates an ant
   * for it. Furthermore, for each  nest, an ant included in the 
   * manager queue is selected and its <CODE>run()</CODE> method
   * is executed. Statistics about the simulated execution of the
   * code are collected in the <CODE>Stats</CODE> object specified
   * through method <CODE>setStats()</CODE>.
   */
  public void evaluate()
  {
   
    /* Execute the initialization rounds */ 
    for (int i=0; i < initRounds; i++) {
      /* Generate requests */
      requestGenerator.generateRequests();
      /* Each nest executes the code of an ant in its queue */
      peernet.executeRound();
    }
    
    /* Execute the regular rounds */ 
    for (int i=0; i < regularRounds; i++) {
      /* Collects statistics */
      monitor.tick();
      /* Generate requests */
      requestGenerator.generateRequests();
      /* Each nest executes the code of an ant in its queue */
      peernet.executeRound();
    }

    /* Execute the final rounds */ 
    for (int i=0; i < finalRounds; i++) {
      /* Collects statistics */
      monitor.tick();
      /* Each nest executes the code of an ant in its queue */
      peernet.executeRound();
    }
  }

  /**
   *  Analyze the scenario object by displaying information about it.
   *  The behavior of this method is delagated to an instance of
   *  <CODE>ScenarioAnalyzer</CODE>.
   */
  public void analyze(Stats stats)
  {
    analyzer.analyze(stats, peernet);
  }
  
  
} // END ScenarioImpl

