/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.impl;

import anthill.factory.Factory;
import anthill.util.*;
import antsim.*;

/**
 *  This class implements an experiment in which the evolution through
 *  time of a simulation is shown. The experiment is repeated for a
 *  number of times specified by parameter <CODE>Repetitions</CODE>.
 *  Each experiment is composed of a number of iterations of the
 *  configured scenario; this number is specified using the parameter
 *  <CODE>Iterations</CODE>. When evaluated, this experiment returns
 *  statistics for each iteration of each repetitions, arranged
 *  as an array of arrays of statistcs objects.
 *   
 *  @author Alberto Montresor
 *  @version $Revision: 1.5 $
 */
public class TimeExperiment
implements Experiment
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Property identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  String name of the Java property used to determine whether the
   *  the analyisis of the footprint is activated.
   */
  public static final String PROP_FOOTPRINT = "Footprint";

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  String name of the parameter used to determine the number of
   *  repetitions to be executed.
   */
  public static final String PAR_REPETITIONS = "Repetitions";
  
  /**
   *  String name of the parameter used to determine the number of
   *  iterations to be executed.
   */
  public static final String PAR_ITERATIONS = "Iterations";
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   *  Number of iterations composing an experiment. During a single 
   *  iteration, method <CODE>evaluate</CODE> of the scenario associated
   *  to this experiment is evaluated.
   */
  protected int iterations;
  
  /** 
   *  Number of repetions of this experiment. The experiment is
   *  repeated a certain number of times, in order to obtain mean
   *  values.
   */
  protected int repetitions;
  
  /** 
   *  If true, methods for the analysis of experiment footprint are 
   *  invoked 
   */
  protected boolean footprint;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Factory used to instantiate objects */
  protected Factory factory;

  /** Scenario on which evaluate the experiment */
  protected Scenario scenario;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates a new time experiment. Configuration parameters are
   *  read through the specified factory object.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public TimeExperiment(Factory factory, String name)
  {
    this.factory = factory;
    clear(); 

    /* Read system properties */
    footprint = Boolean.getBoolean(PROP_FOOTPRINT);
  }

  /**
   *  Restores the object to its initial state. 
   */
  public void clear()
  {
    /* Read configuration parameters */
    iterations = factory.intValue(NAME + "." +PAR_ITERATIONS);
    repetitions = factory.intValue(NAME + "." +PAR_REPETITIONS);

    /* Creates local variables */
    if (scenario == null)
      scenario = (Scenario) factory.get(Scenario.NAME);
    else
      scenario.clear();
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Experiment
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Executes the evaluation of the current ant algorithm. Returns an
   *  array of array of <CODE>Stats</CODE> objects containing the 
   *  statistics collected during the simulation. 
   */
  public Object evaluate()
  {
    long time, elapsed;
    
    /*  
     *  Create the statistics object needed to collect
     *  statistics about the number of repetitions
     *  of the simulation.
     */
    Stats[][] results = new Stats[repetitions][iterations];
      
    for (int j=0; j < repetitions; j++) {

      for (int i=0; i < iterations; i++) {
        
        results[j][i] = new Stats();
        results[j][i].add("Repetition", j+1);

        /* Compute time for statistics */
        time = System.currentTimeMillis();
        
        /* Updates statistics and evaluate the scenario */
        results[j][i].add("Iteration", i+1);
        scenario.setStats(results[j][i]);
        scenario.evaluate();
      
        /* Compute elapsed time */
        elapsed = (System.currentTimeMillis()-time);
        time = System.currentTimeMillis();
        results[j][i].add("Simulation time", elapsed);

        /* For debug purposes */
        if (footprint) {
          // Log.println(Analyzer.printClasses(this));
          results[j][i].add("Footprint", Analyzer.footprint(this));
          elapsed = (System.currentTimeMillis()-time);
          results[j][i].add("Analyzer time", elapsed);
        }

        /* Print statistics */
        scenario.analyze(results[j][i]);
      }
      Log.println("\n-----------------------------------------------------");
      scenario.clear();
    }
    return results;
  }

} // END TimeExperiment

