/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.nestor;

import java.io.IOException;

import anthill.*;
import anthill.nest.AntDescriptor;


/**
 *  Class LocalAntView is a simple implementation of interface 
 *  <CODE>AntView</CODE>, which implements methods included in
 *  it by invoking the opportune methods on the local nest.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.6 $
 */
public class LocalAntView
implements AntView
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
 
  /** Reference to the nest object hosting this view */
  private LocalNest nest;
  
  /** The descriptor of the ant currently associated with this view */
  private AntDescriptor desc;

  /** True if the resource method move() has been invoked successfully */
  private boolean moved;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates an AntView and stores in it the references to
   *  which the operations are delegated.
   *  <p>
   *  The constructor has package visibility to avoid
   *  instances not builded through the appropriate factory.
   *
   *  @param storage  the resource database
   *  @param gate     the communication handler
   *  @param desc     the descriptor of the ant to which this AntView
   *                  is associated
   */
  LocalAntView(LocalNest nest, AntDescriptor desc)
  {
    this.nest = nest;
    this.desc = desc;
    moved = false;
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for requesting general information
  ////////////////////////////////////////////////////////////////////////////////////////////

  /*  
   *  Javadoc comment inherited by interface anthill.AntView
   *  Asks the nest for its identifier
   */
  public NestId getNestId()
  {
    return nest.getNestId();
  }
  
  /*  
   *  Javadoc comment inherited by interface anthill.AntView
   *  Asks the associated ant descriptor for the nest identifier
   *  of the last visited nest.
   */
  public NestId getLastVisited()
  {
    return desc.getLastVisited();
  }

  /*  
   *  Javadoc comment inherited by interface anthill.AntView
   *  Asks the associated nest descriptor for the remaining
   *  time-to-live.
   */
  public int getTTL()
  {
    return desc.getTTL();
  }

  /*
   *  Javadoc comment inherited by interface anthill.AntView
   *  Asks the associated ant descriptor for the ant reference
   */
  public Ant getAnt()
  {
    return desc.getAnt();
  }

  /**
   *  Creates a new ant identifier.
   */
  public AntId createAntId()
  {
    return new LocalAid();
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for ant movement and creation
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  FIXME: comment.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Manager</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public void addAnt(Ant ant)
  {
    if (ant == null) 
      throw new NullPointerException("The ant to execute cannot be null");
    AntDescriptor newdesc = 
      new AntDescriptor(nest.getNestId(), ant, desc.getTTL());
    LocalAntView view = new LocalAntView(nest, newdesc);
    ant.init(view);
    nest.getManager().add(view);
  }

  /**
   *  Moves an ant to the specified nest. In this particular implementation,
   *  an ant can perform at most one move operation when in a nest. This 
   *  restriction is imposed in order to avoid unbounded ant replication.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Gate</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   *
   *  @param id the id of the destination nest
   *  @throws IOException if the ant cannot move to the
   *    destination nest for communication problems.
   */
  public void move(NestId id)
    throws IOException
  {
    nest.getGate().send(id, desc.getAnt(), desc.getTTL());
    /*
     *  Note that moved is updated only if we are
     *  sure that the send operation has not
     *  raised an exception.
     */
    moved = true;
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for neighbour management
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Add the new neighbor information to the <CODE>Gate</CODE> object contained
   *  in the current nest. FIXME: correct implementation by removing addAntNeighbour
   *  and adding the method to Gate.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Gate</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public void addNeighbour(NestId id)
    throws IOException
  {
    nest.addAntNeighbour(id);
  }
        
  /**
   *  FIXME: Implement.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Gate</CODE> interface currently used in the current nest.
   *  Please refer to the class documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public void removeNeighbour(NestId id)
  {
    // nest.removeAntNeighbour(id);
  }
        
  
  /**
   *  Ants invoke this method to obtain the list of nest neighbors
   *  known to the current nest. FIXME: correct implementation to refer to Gate
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Gate</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public Neighbour[] getNeighbours()
  {
    return nest.getNeighbours();
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for pheromone management
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns a reference to the resource manager identified by the given name.
   *  Returns null if no resource manager is associated with the given name
   *  to this ant.
   */
  public ResourceManager getResourceManager(String name)
  {
    return nest.getResourceManager(name, null);
  }
  
  /**
   *  Returns a reference to the resource manager identified by the given name,
   *  if present. Otherwise, creates a new resource manager, using name as 
   *  lookup identifier in the factory object to obtain the current configuration.
   */
  public ResourceManager createResourceManager(String name)
    throws ResourceManagerException
  {
    return nest.createResourceManager(name, null);
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for resource management
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  This method is invoked by ants to notify the nest that
   *  the ant associated to this AntView, created to satisfy
   *  a request, is returning the specified set of resources.
   *
   *  @param request the request to be performed
   *  @param resources  the resources satisfying the query
   */
  public boolean returnResources(RequestId rid, Resource[] resources)
  {
    return nest.getRequestSet().returnResources(rid, resources);
  }

} // END LocalAntView

