/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.nestor;

import java.io.IOException;

import anthill.*;
import anthill.factory.Factory;
import anthill.nest.*;

/**
 *  Class <CODE>LocalGate</CODE> is the implementation of <CODE>Gate</CODE>
 *  to be used during off-line simulations. Communication among different
 *  <CODE>LocalGate</CODE>s is based on local method invocations.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.3 $
 */
public class LocalGate
implements Gate
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Reference to the object listening for ants */
  private AntListener listener;

  /** Identifier of this nest */
  private LocalNid me;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  
  /**
   *  Initializes a new <CODE>LocalGate</CODE> by creating
   *  a new local nest identifier.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public LocalGate(Factory factory, String name)
  {
    me = new LocalNid(this);
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Gate
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the identifier of the <CODE>Nest</CODE> to which this
   *  gate is associated.
   *
   */
  public NestId getNestId()
  {
    return me;
  }

  
  /**
   *  Sets the ant listener, i.e. the object that will
   *  accept the ants coming from other nests.
   *
   *  @param listener the listener object that will
   *    accepts the ants coming from other nests
   */
  public void setAntListener(AntListener listener)
  {
    this.listener = listener;
  }
  
  
  /**
   *  Sends an ant to the specified nest. Both the nest
   *  identifier and the ant must be serializable or
   *  externalizable.
   *
   * @exception ClassCastException
   *   raised if the nest identifier does not belong to the
   *   expected class
   * @exception IOException
   *   raised if problems have occurred during serialization
   *   and sending
   */
  public void send(NestId dest, Ant ant, int ttl) 
    throws ClassCastException, IOException
  {
    // anthill.util.Log.println("Before ttl check" + desc.getTTL());
    if (ttl > 0) {
      LocalNid ldest = (LocalNid) dest;
      ldest.getGate().deliver(me, ant, ttl);
    }
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Clearable
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
    listener = null;
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Private methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  This method is invoked by other <CODE>LocalGate</CODE> in order to queue
   *  ant in this gate object.
   */
  private void deliver(LocalNid sender, Ant ant, int ttl)
    throws IOException
  {
    // anthill.util.Log.println("Delivered to " + me + " from " + sender);
    ttl--;
    AntDescriptor desc = new AntDescriptor(sender, ant, ttl);
    if (listener == null) {
      /*
       *  No listener is set; we assume that this node has leaved
       *  the system and we return an exception 
       */
      throw new IOException();
    } else {
      /* 
       *  Listener is set; we deliver the ant
       */
      listener.deliver(desc);
    }
  }
  
} // END LocalGate

