/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.nestor;

import java.util.*;

import anthill.*;
import anthill.factory.Factory;
import anthill.nest.*;
import anthill.storage.Trail;
import antsim.SimulatedNest;

/**
 *  A Nest class collects the components of a Nest site: (i) a AntView
 *  object, which is the view of a Nest for an Ant; (ii) a Gate object,
 *  which manage the communication between nests; (iii) an Manager,
 *  i.e. the object which manage the activities performed by different agents.
 *  <p>
 *  Implementation note: this implementation is supposed to be used as
 *  simple implementation of <CODE>Nest</CODE> in the off-line
 *  environment. Here, we assume that the Manager object used is single-threaded;
 *  in this way, we avoid the costs of synchronized methods.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.5 $
 */
public class LocalNest
implements SimulatedNest, AntListener
{
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   *  String name of the parameter used to determine the ttl used by these nests.
   */
  public static final String PAR_TTL = "TTL";

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Time-to-live used to initialize ants */
  private int ttl;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Factory used to instantiate objects */
  private Factory factory;
  
  /** Ant factory used to generate ants for request */
  private AntFactory antf;
  
  /** Communication handler to be used to communicate with other nests */
  private Gate gate;
  
  /** Hash table mapping request to response listeners */
  private RequestSet requests;

  /** Thread manager needed to be used to manage concurrent agents */
  private Manager manager;

  /** Back trail information */
  private Trail trail;
  
  /** Neighbours hash table, with NestId as key (FIXME move in Gate) */
  private HashMap map;

  /** Neighbours array (FIXME fixed, should be configurable) */
  private Neighbour[] neighbours;

  /** Resource manager map */
  HashMap managerMap;
  
  /** Pheromone object managed by ants */
  private Object pheromone;
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Constructs a new nest.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public LocalNest(Factory factory, String name)
  {
    /* Update factories */
    this.factory = factory;
    clear();
  }

        
  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
    /*
     *  Read parameter TTL from the factory object; if the parameter is found, 
     *  the value are updated. Otherwise, the initialization parameter is still valid.
     */
    this.ttl = factory.intValue(NAME + "." +PAR_TTL);
    
    /* 
     *  Obtain implementations of basic interfaces. Note that in this class
     *  we do not manage the possibility of changing implementations of
     *  the basic components of nests at runtime. So, we do not need to
     *  maintain the versions of the implementations classes stored here.
     */
    
    /* Gate */
    if (gate == null) {
      gate = (Gate) factory.get(Gate.NAME);
    } else {
      gate.clear();
    }
    gate.setAntListener(this);
    
    /* RequestSet */
    if (requests == null) {
      requests = (RequestSet) factory.get(RequestSet.NAME);
    } else {
      requests.clear();
    }
    
    /* Manager */
    if (manager == null) {
      manager = (Manager) factory.get(Manager.NAME);
    } else {
      manager.clear();
    }
    
    /* Ant factory */
    if (antf == null) {
      antf = (AntFactory) factory.get(AntFactory.NAME);
    } else {
      antf.clear();
    }
    
    if (trail != null) {
      trail.clear();
    }
    
    /* Pheromone */
    pheromone = null;
    
    /* Storage management */ 
    if (managerMap == null) {
      managerMap = new HashMap();
    } else {
      managerMap.clear();
    }
    
    // FIXME Move this part into Gate
    if (map == null) {
      map = new HashMap();
    } else {
      map.clear();
    }
    neighbours = null;
    
  }
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from AntListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Receive an agent from outside and deliver it to the
   *  to the activity manager.
   */
  public void deliver(AntDescriptor desc)
  {
    AntView view = new LocalAntView(this, desc); 
    manager.add(view);
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Nest
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Forces the nest to exit. 
   */
  public void leave()
  {
    gate.setAntListener(null);
  }

 
  /**
   *  Return the identifier of this Nest
   */
  public NestId getNestId()
  {
    return gate.getNestId();
  }
  
  /**
   *  Return the Manager associated to this Nest
   */
  public Manager getManager()
  {
    return manager;
  }

  /**
   *  Accepts a request from the nest user. Possible responses will be
   *  returned to the specified listener. One or more ants are generated
   *  and assigned to the request.
   *  Accept a resource request originated by an anthill client.
   *  The resource to be searched is identified by name. The method
   *  returns immediately. When the resource is found, a Resource
   *  object will be returned to the specified resource listener.
   *  <p>
   *  Implementation note: If the resource can be found locally, 
   *  the listener is immediately contacted. Otherwise, a new 
   *  <CODE>Ant</CODE> is generated and is delivered to the ant 
   *  manager.
   *
   *  @param request the request to be performed
   *  @param listener The listener object which will be notified
   *    when the request is completed.
   */
  public void request(RequestId rid, Object request, ResponseListener listener)
  {
    /* Creates a set of ants and insert them in the local nest */
    Ant[] ants = antf.getAnts(rid, request, gate.getNestId());
    for (int i=0; i < ants.length; i++) {
      AntDescriptor desc = new AntDescriptor(gate.getNestId(), ants[i], ttl);
      AntView view = new LocalAntView(this, desc);
      ants[i].init(view);
      manager.add(view);
    } 
    if (listener != null) {
      requests.add(rid, listener);
    }
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Package methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Returns a reference to the resource manager identified by the given name.
   */
  public ResourceManager getResourceManager(String name, String species)
  {
    return (ResourceManager) managerMap.get(name);
  }
  
  /**
   *  Returns a reference to the resource manager identified by the given name.
   */
  public ResourceManager createResourceManager(String name, String species)
    throws ResourceManagerException
  {
    ResourceManager rm = (ResourceManager) managerMap.get(name);
    if (rm == null) {
      try { 
        rm = (ResourceManager) factory.get(name);
        managerMap.put(name, rm);
      } catch (Exception e) { 
        throw new ResourceManagerException(e); 
      }
    }
    return rm;
  }
  
  /**
   *  Add the identifier of a new neighbour nest to the set of 
   *  neighbours known to this <CODE>Nest</CODE>, on behalf of 
   *  an Nest client. 
   *
   *  @param aid neighbour identifier
   */
  public void addClientNeighbour(NestId aid)
  {
    map.put(aid, new Neighbour(aid));
    neighbours = null;
  }
  
  
  /**
   *  Add the identifier of a new neighbour nest to the set of 
   *  neighbours known to this <CODE>Nest</CODE>, on behalf of 
   *  an ant. 
   *
   *  @param aid neighbour identifier
   */
  public void addAntNeighbour(NestId aid)
  {
    map.put(aid, new Neighbour(aid));
    neighbours = null;
  }

  
  /**
   * Remove a neighbour identifier from the set of neighbour
   * @param aid neighbour identifier
   */
  public void removeNeighbour(NestId aid)
  {
    map.remove(aid);
    neighbours = null;
  }
  
  /**
   * Return the neighbours known to this Nest
   */
  public Neighbour[] getNeighbours()
  {
    if (neighbours == null) {
      neighbours = new Neighbour[map.size()];
      Iterator iterator = map.values().iterator();
      for (int i=0; i < neighbours.length; i++) 
        neighbours[i] = (Neighbour) iterator.next();
    }
    return neighbours;
  }
  

  
  /**
   *  Return the <CODE>Gate</CODE> object used by this <CODE>Nest</CODE>
   */
  Gate getGate()
  {
    return gate;
  }

  /**
   *  Return the <CODE>RequestSet</CODE> object used by this <CODE>Nest</CODE>
   */
  RequestSet getRequestSet()
  {
    return requests;
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Package methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  FIXME .... 
   */
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();
    buffer.append("[Nest: ");
    buffer.append(trail);
    buffer.append(", ");
    buffer.append(requests);
    buffer.append("]]");
    return buffer.toString();
  }

} // END LocalNest
