/* Generated by Together */

package jxtaimpl;

import java.io.*;

import net.jxta.document.Advertisement;
import net.jxta.endpoint.*;
import net.jxta.id.ID;
import net.jxta.peergroup.PeerGroup;
import net.jxta.pipe.*;
import net.jxta.protocol.*;
import net.jxta.service.Service;

import anthill.*;
import anthill.nest.*;

/**
 * This is a Jxta-Enabled-Service. This class implements the Gate interface and is meant for 
 * receiving ants coming from other nests. 
 * @author Francesco Russo (frusso@cs.unibo.it)
 * @stereotype implementationClass
 */
public class GateService implements Gate, Service 
{
  /**
  * The init method will be invoked by the Jxta platform during the AnthillGroup initialization process.
  * PeerGroup grp refers to the AnthillGroup instance, while Advertisement adv refers to the GateService Module Impl Adv.
  * The steps taken here are decribed in the "GateService StartUp" sequence diagram provided with the javadocs.
  * The GateServiceServer and the GateServiceReceiver threads are started in the startApp method.
  * All the gsReceiver and gsServer settings concerning priorities and other thread related things 
  * (i.e. is a deamon or not) are performed here.
  */
  public void init(PeerGroup grp, ID id, Advertisement adv) 
  {
	//* do some settings
	anthillGrp = grp;
	netPipeService = ((AnthillGroup)anthillGrp).getNetPipeService();
	nest = ((AnthillGroup)anthillGrp).getNest();
	listener = nest;
	gsAdvertisement = (ModuleImplAdvertisement) adv;
	nestId = nest.getNestId();
  
	//* creating the buffer 
	gsBuffer = new GateServiceBuffer();
	
	//* creating the receiver and setting the buffer
	gsReceiver = new GateServiceReceiver();
	gsReceiver.setBuffer( gsBuffer );
	gsReceiver.setMsgAntTag( MSG_ANT_TAG );
	
	//* obtaining the pipe advertisement
	gsPipeAdv = ((AnthillGroup)anthillGrp).getGsPipeAdv();
	
	//* passing it to the receiver instance
	gsReceiver.setInputPipeAdv( gsPipeAdv );
	gsReceiver.setInputPipe( createGateServiceInputPipe( gsPipeAdv ) );
	
	//* create the server and do all the settings
	gsServer = new GateServiceServer();
	gsServer.setBuffer( gsBuffer );
	gsServer.setNest( nest );
	
	gsReceiver.setPriority( Thread.MAX_PRIORITY );
	gsServer.setPriority( Thread.MAX_PRIORITY );
	
	System.out.println("--- GateService initialized");
  }

  /**
  * The startApp method will start the GateServiceServer and the GateServiceReceiver threads.
  * This method does not need to be invoked, the Jxta platform will do that.
  */
  public int startApp(String[] args) 
  {
	gsServer.start();
	gsReceiver.start();
	
	System.out.println("--- GateService started!");
	return(0);
  }

  /**
  * Method for shutting the service down. 
  */
  public void stopApp() 
  {
  }

  /**
  * This method returns the Service interface  
  */
  public Service getInterface()
  {
	return this;
  }

  /**
  * This method returns the GateService ModImplAdvertisement 
  */
  public Advertisement getImplAdvertisement() 
  {
	return gsAdvertisement;
  }

  /**
  * Returns the id associated with the nest this Gate belongs to
  */
  public NestId getNestId() 
  {
	return nestId;
  }

  /**
  * This is the method which allows ants to be moved from nest to nest.
  * Its arguments are the destination nest id, the ant to be sent, and a Time To Live value (number of max hops).
  * The ant is sent just if the TTL value will be > 0
  *
  * @param dest The <code>jxtaimpl.ids.NestId</code> ID of the destination nest
  * @param ant The <code>Ant</code> to be sent
  * @param ttl This ant's TTL value 
  */
  public void send(NestId dest, Ant ant, int ttl) throws IOException
  {
	if ( ttl > 0)
	{
	  OutputPipe outPipe = null;
	  AntDescriptor aDesc = null;
	  jxtaimpl.ids.NestId destNestId = (jxtaimpl.ids.NestId)dest;
	  
	  System.out.println("--- GateService: send()");
	  
	  ttl = ttl-1; 
	  aDesc = new AntDescriptor( nestId, ant, ttl);     
	  
	  //* let's serialize the AntDescriptor
	  ByteArrayOutputStream baos = new ByteArrayOutputStream();
	  ObjectOutputStream oos = new ObjectOutputStream( baos );
	  oos.writeObject( aDesc );
	  byte[] buffer = baos.toByteArray();
		
	  try
	  {
		//* let's create an output pipe using the destination nest's gate service pipe advertisement
		synchronized( netPipeService )
		{
		  outPipe = netPipeService.createOutputPipe( destNestId.getPipeAdv(), PIPE_CREATION_WAITING_TIME );
		  Message msg = netPipeService.createMessage();       
		
		  //* put in the message instance the Ant under the tag MSG_ANT_TAG
		  MessageElement element = msg.newMessageElement( MSG_ANT_TAG, null, buffer ); 
		  msg.addElement( element );
	  
		  //* send the message
		  outPipe.send( msg );
		  System.out.println("--- GateService: send() done!");
		}
	  }
	  catch(IOException e)
	  {
		System.out.println("--- GateService: send() - Nest unreachable!");
		((jxtaimpl.Nest)nest).removeNeighbour( destNestId );
		System.out.println("--- GateService: send() - NestID removed!");
		throw new IOException();
	  }
	}
  }
  
  /**
  * This method creates the input pipe used by the GateServiceReceiver instance. This pipe has to be
  * passed to the receiver via the setInputPipe(InputPipe pipe) method
  */
  private InputPipe createGateServiceInputPipe( PipeAdvertisement pipeAdv )
  {
	InputPipe pipe = null;
	
	try
	{
	  pipe = netPipeService.createInputPipe( pipeAdv );
	}
	catch (IOException ioe)
	{
	  ioe.printStackTrace();
	  System.exit(1);
	}
	
	return pipe;
  }
  
  /**
  * This method sets a reference to the instance which is willing to accept ants coming from other nests.
  * In the "jxtaimpl" implementation the Nest class implements the AntListener interface, so the AntListener
  * is the Nest instance itself. The GateServiceServer instance has to get this reference.
  * IMPORTANT: with this implementation this method is not useful anymore since the AntListener is set in 
  * the init() method using the reference to the Nest instance.
  */
  public void setAntListener( AntListener antListnr )
  {
	listener = antListnr;
  }
  
  public void clear()
  {
  }

  private Nest nest;
  private PipeService netPipeService;
  private AntListener listener;
  private GateServiceReceiver gsReceiver;
  private GateServiceBuffer gsBuffer;
  private GateServiceServer gsServer;
  private PeerGroup anthillGrp;
  private ModuleImplAdvertisement gsAdvertisement;
  private NestId nestId;
  private PipeAdvertisement gsPipeAdv;
  
  private long PIPE_CREATION_WAITING_TIME = 10000;
  private String MSG_ANT_TAG = "ANTHILL-GROUP-ANT-TAG";
}
